import datetime as dt
from io import BytesIO, StringIO
import json
from nselib.libutil import *
from nselib.constants import *


def securities_available_for_trading(trade_date):
    """
    get securities available for trading
    trade_date: 17-03-2022' ('dd-mm-YYYY') format
    :return: pandas dataframe
    """
    month = dt.datetime.strptime(trade_date, '%d-%m-%Y').strftime('%b').upper()
    year = dt.datetime.strptime(trade_date, '%d-%m-%Y').strftime('%Y')
    date_str = dt.datetime.strptime(trade_date, '%d-%m-%Y').strftime('%d%m%Y')
    origin_url = "https://www.nseindia.com/all-reports-debt"
    url = f"https://nsearchives.nseindia.com/content/historical/WDM/{year}/{month}/wdmlist_{date_str}.csv"
    file_chk = nse_urlfetch(url, origin_url=origin_url)
    if file_chk.status_code != 200:
        raise FileNotFoundError(f" No data available")
    try:
        data_df = pd.read_csv(BytesIO(file_chk.content))
    except Exception as e:
        raise FileNotFoundError(f' Equity List not found :: NSE error : {e}')
    return data_df


# if __name__ == '__main__':
#     data = securities_available_for_trading('02-12-2025')
#     print(data)



import datetime as dt
import zipfile
from nselib.derivatives.get_func import *


def future_price_volume_data(symbol: str, instrument: str, from_date: str = None, to_date: str = None,
                             period: str = None):
    """
    get contract wise future price volume data set.
    :param instrument:  FUTIDX/FUTSTK
    :param symbol: symbol eg: 'SBIN' / 'BANKNIFTY'
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,
                            '1W': for last 7 days data,
                            '1M': from last month same date,
                            '3M': last 3 month data
                            '6M': last 6 month data}
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    symbol, instrument = cleaning_nse_symbol(symbol=symbol), instrument.upper()
    if instrument not in ['FUTIDX', 'FUTSTK']:
        raise ValueError(f'{instrument} is not a future instrument')

    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=future_price_volume_data_column)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 90:
            end_date = (from_date + dt.timedelta(90)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_future_price_volume_data(symbol=symbol, instrument=instrument,
                                               from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(91)
        load_days = (to_date - from_date).days
        if nse_df.empty:
            nse_df = data_df
        else:
            nse_df = pd.concat([nse_df, data_df], ignore_index=True)
    return nse_df


def option_price_volume_data(symbol: str, instrument: str, option_type: str = None, from_date: str = None,
                             to_date: str = None, period: str = None):
    """
    get contract wise option price volume data set. more than 90 days will take more time to collect data.
    :param option_type: PE/CE
    :param instrument:  OPTIDX/OPTSTK
    :param symbol: symbol eg: 'SBIN' / 'BANKNIFTY'
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,
                            '1W': for last 7 days data,
                            '1M': from last month same date,
                            '3M': last 3 month data
                            '6M': last 6 month data}
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    symbol, instrument = cleaning_nse_symbol(symbol=symbol), instrument.upper()
    if instrument not in ['OPTIDX', 'OPTSTK']:
        raise ValueError(f'{instrument} is not a future instrument')

    if option_type and option_type not in ['PE', 'CE']:
        raise ValueError(f'{option_type} is not a valid option type')

    option_type = [option_type] if option_type else ['PE', 'CE']
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=future_price_volume_data_column)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 90:
            end_date = (from_date + dt.timedelta(90)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        for opt_typ in option_type:
            data_df = get_option_price_volume_data(symbol=symbol, instrument=instrument, option_type=opt_typ,
                                                   from_date=start_date, to_date=end_date)
            if nse_df.empty:
                nse_df = data_df
            else:
                nse_df = pd.concat([nse_df, data_df], ignore_index=True)
        from_date = from_date + dt.timedelta(91)
        load_days = (to_date - from_date).days

    return nse_df


def fno_bhav_copy(trade_date: str):
    """
    new CM-UDiFF Common NSE future option bhav copy from 2018 on wards
    :param trade_date: eg:'20-06-2023'
    :return: pandas Data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    url = 'https://nsearchives.nseindia.com/content/fo/BhavCopy_NSE_FO_0_0_0_'
    payload = f"{str(trade_date.strftime('%Y%m%d'))}_F_0000.csv.zip"
    request_bhav = nse_urlfetch(url + payload)
    bhav_df = pd.DataFrame()
    if request_bhav.status_code == 200:
        zip_bhav = zipfile.ZipFile(BytesIO(request_bhav.content), 'r')
        for file_name in zip_bhav.filelist:
            if file_name:
                bhav_df = pd.read_csv(zip_bhav.open(file_name))
    elif request_bhav.status_code == 403:
        url2 = "https://www.nseindia.com/api/reports?archives=" \
             "%5B%7B%22name%22%3A%22F%26O%20-%20Bhavcopy(csv)%22%2C%22type%22%3A%22archives%22%2C%22category%22" \
             f"%3A%22derivatives%22%2C%22section%22%3A%22equity%22%7D%5D&date={str(trade_date.strftime('%d-%b-%Y'))}" \
             f"&type=equity&mode=single"
        request_bhav = nse_urlfetch(url2)
        if request_bhav.status_code == 200:
            zip_bhav = zipfile.ZipFile(BytesIO(request_bhav.content), 'r')
            for file_name in zip_bhav.filelist:
                if file_name:
                    bhav_df = pd.read_csv(zip_bhav.open(file_name))
        elif request_bhav.status_code == 403:
            raise FileNotFoundError(f' Data not found, change the date...')
    # bhav_df = bhav_df[['INSTRUMENT', 'SYMBOL', 'EXPIRY_DT', 'STRIKE_PR', 'OPTION_TYP', 'OPEN', 'HIGH', 'LOW',
    #                    'CLOSE', 'SETTLE_PR', 'CONTRACTS', 'VAL_INLAKH', 'OPEN_INT', 'CHG_IN_OI', 'TIMESTAMP']]
    return bhav_df


def participant_wise_open_interest(trade_date: str):
    """
    get FII, DII, Pro, Client wise participant OI data as per traded date
    :param trade_date: eg:'20-06-2023'
    :return: pandas Data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    url = f"https://nsearchives.nseindia.com/content/nsccl/fao_participant_oi_{str(trade_date.strftime('%d%m%Y'))}.csv"
    # payload = f"{str(for_date.strftime('%d%m%Y'))}.csv"
    file_chk = nse_urlfetch(url)
    if file_chk.status_code == 404:
        url = f"https://archives.nseindia.com/content/nsccl/fao_participant_oi_{str(trade_date.strftime('%d%m%Y'))}.csv"
        file_chk = nse_urlfetch(url)
    if file_chk.status_code != 200:
        raise FileNotFoundError(f" No data available for : {trade_date}")
    try:
        # data_df = pd.read_csv(url, engine='python', sep=',', quotechar='"', on_bad_lines='skip', skiprows=1)
        data_df = pd.read_csv(BytesIO(file_chk.content), on_bad_lines='skip', skiprows=1)
    except:
        data_df = pd.read_csv(BytesIO(file_chk.content), on_bad_lines='skip', skiprows=1)
        data_df.drop(data_df.tail(1).index, inplace=True)
        data_df.columns = [name.replace('\t', '') for name in data_df.columns]
    return data_df


def participant_wise_trading_volume(trade_date: str):
    """
    get FII, DII, Pro, Client wise participant volume data as per traded date
    :param trade_date: eg:'20-06-2023'
    :return: pandas Data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    url = f"https://nsearchives.nseindia.com/content/nsccl/fao_participant_vol_{str(trade_date.strftime('%d%m%Y'))}.csv"
    # payload = f"{str(for_date.strftime('%d%m%Y'))}.csv"
    file_chk = nse_urlfetch(url)
    if file_chk.status_code != 200:
        raise FileNotFoundError(f" No data available for : {trade_date}")
    try:
        data_df = pd.read_csv(BytesIO(file_chk.content), on_bad_lines='skip', skiprows=1)
    except Exception:
        data_df = pd.read_csv(BytesIO(file_chk.content), engine='c', sep=',', quotechar='"',
                              on_bad_lines='skip', skiprows=1)
        data_df.drop(data_df.tail(1).index, inplace=True)
        data_df.columns = [name.replace('\t', '') for name in data_df.columns]
    return data_df


def fii_derivatives_statistics(trade_date: str):
    """
    get FII derivatives statistics as per the traded date provided
    :param trade_date: eg:'20-06-2023'
    :return: pandas dataframe
    """
    t_date = pd.to_datetime(trade_date, format='%d-%m-%Y')
    trade_date = t_date.strftime('%d-%b-%Y')
    url = f"https://nsearchives.nseindia.com/content/fo/fii_stats_{trade_date}.xls"
    file_chk = nse_urlfetch(url)
    if file_chk.status_code != 200:
        raise FileNotFoundError(f" No data available for : {trade_date}")
    try:
        bhav_df = pd.read_excel(BytesIO(file_chk.content), skiprows=3, skipfooter=10).dropna()
        bhav_df.columns = ['fii_derivatives', 'buy_contracts', 'buy_value_in_Cr', 'sell_contracts', 'sell_value_in_Cr',
                           'open_contracts', 'open_contracts_value_in_Cr']
    except Exception as e:
        raise FileNotFoundError(f' FII derivatives statistics not found for : {trade_date} :: NSE error : {e}')
    return bhav_df


def expiry_dates_future():
    """
    get the future and option expiry dates as per stock or index given
    :return: list of dates
    """
    origin_url = "https://www.nseindia.com/option-chain"
    payload = nse_urlfetch(f'https://www.nseindia.com/api/option-chain-contract-info?symbol=TCS',
                           origin_url=origin_url).json()
    return payload['expiryDates']


def expiry_dates_option_index():
    """
    get the future and option expiry dates as per stock or index given
    :return: dictionary
    """
    # data_df = pd.DataFrame(columns=['index', 'expiry_date'])
    data_dict = {}
    for ind in indices_list:
        origin_url = "https://www.nseindia.com/option-chain"
        payload = nse_urlfetch(f'https://www.nseindia.com/api/option-chain-contract-info?symbol={ind}',
                               origin_url=origin_url).json()
        data_dict.update({ind: payload['expiryDates']})
    return data_dict


def nse_live_option_chain(symbol: str, expiry_date: str = None, oi_mode: str = "full"):
    """
    get live nse option chain.
    :param symbol: eg:SBIN/BANKNIFTY
    :param expiry_date: '20-06-2023'
    :param oi_mode: eg: full/compact
    :return: pands dataframe
    """

    if expiry_date:
        exp_date = pd.to_datetime(expiry_date, format='%d-%m-%Y')
        expiry_date = pd.to_datetime(exp_date, format='%d-%m-%Y').strftime('%d-%b-%Y')
    payload = get_nse_option_chain(symbol, expiry_date).json()
    if oi_mode == 'compact':
        col_names = ['Fetch_Time', 'Symbol', 'Expiry_Date', 'CALLS_OI', 'CALLS_Chng_in_OI', 'CALLS_Volume', 'CALLS_IV',
                     'CALLS_LTP', 'CALLS_Net_Chng', 'Strike_Price', 'PUTS_OI', 'PUTS_Chng_in_OI', 'PUTS_Volume',
                     'PUTS_IV', 'PUTS_LTP', 'PUTS_Net_Chng']
    else:
        col_names = ['Fetch_Time', 'Symbol', 'Expiry_Date', 'CALLS_OI', 'CALLS_Chng_in_OI', 'CALLS_Volume', 'CALLS_IV',
                     'CALLS_LTP', 'CALLS_Net_Chng', 'CALLS_Bid_Qty', 'CALLS_Bid_Price', 'CALLS_Ask_Price',
                     'CALLS_Ask_Qty', 'Strike_Price', 'PUTS_Bid_Qty', 'PUTS_Bid_Price', 'PUTS_Ask_Price', 'PUTS_Ask_Qty',
                     'PUTS_Net_Chng', 'PUTS_LTP', 'PUTS_IV', 'PUTS_Volume', 'PUTS_Chng_in_OI', 'PUTS_OI']

    oi_data = pd.DataFrame(columns=col_names)

    oi_row = {'Fetch_Time': None, 'Symbol': None, 'Expiry_Date': None, 'CALLS_OI': 0, 'CALLS_Chng_in_OI': 0, 'CALLS_Volume': 0,
              'CALLS_IV': 0, 'CALLS_LTP': 0, 'CALLS_Net_Chng': 0, 'CALLS_Bid_Qty': 0, 'CALLS_Bid_Price': 0,
              'CALLS_Ask_Price': 0, 'CALLS_Ask_Qty': 0, 'Strike_Price': 0, 'PUTS_OI': 0, 'PUTS_Chng_in_OI': 0,
              'PUTS_Volume': 0, 'PUTS_IV': 0, 'PUTS_LTP': 0, 'PUTS_Net_Chng': 0, 'PUTS_Bid_Qty': 0,
              'PUTS_Bid_Price': 0, 'PUTS_Ask_Price': 0, 'PUTS_Ask_Qty': 0}

    # print(expiry_date)
    for m in range(len(payload['records']['data'])):
        if not expiry_date or (payload['records']['data'][m]['expiryDates'] == expiry_date):
            try:
                oi_row['Expiry_Date'] = payload['records']['data'][m]['expiryDates']
                oi_row['CALLS_OI'] = payload['records']['data'][m]['CE']['openInterest']
                oi_row['CALLS_Chng_in_OI'] = payload['records']['data'][m]['CE']['changeinOpenInterest']
                oi_row['CALLS_Volume'] = payload['records']['data'][m]['CE']['totalTradedVolume']
                oi_row['CALLS_IV'] = payload['records']['data'][m]['CE']['impliedVolatility']
                oi_row['CALLS_LTP'] = payload['records']['data'][m]['CE']['lastPrice']
                oi_row['CALLS_Net_Chng'] = payload['records']['data'][m]['CE']['change']
                if oi_mode == 'full':
                    oi_row['CALLS_Bid_Qty'] = payload['records']['data'][m]['CE']['buyQuantity1']
                    oi_row['CALLS_Bid_Price'] = payload['records']['data'][m]['CE']['buyPrice1']
                    oi_row['CALLS_Ask_Price'] = payload['records']['data'][m]['CE']['sellPrice1']
                    oi_row['CALLS_Ask_Qty'] = payload['records']['data'][m]['CE']['sellQuantity1']
            except KeyError:
                oi_row['CALLS_OI'], oi_row['CALLS_Chng_in_OI'], oi_row['CALLS_Volume'], oi_row['CALLS_IV'], oi_row[
                    'CALLS_LTP'], oi_row['CALLS_Net_Chng'] = 0, 0, 0, 0, 0, 0
                if oi_mode == 'full':
                    oi_row['CALLS_Bid_Qty'], oi_row['CALLS_Bid_Price'], oi_row['CALLS_Ask_Price'], oi_row[
                        'CALLS_Ask_Qty'] = 0, 0, 0, 0
                pass

            oi_row['Strike_Price'] = payload['records']['data'][m]['strikePrice']

            try:
                oi_row['PUTS_OI'] = payload['records']['data'][m]['PE']['openInterest']
                oi_row['PUTS_Chng_in_OI'] = payload['records']['data'][m]['PE']['changeinOpenInterest']
                oi_row['PUTS_Volume'] = payload['records']['data'][m]['PE']['totalTradedVolume']
                oi_row['PUTS_IV'] = payload['records']['data'][m]['PE']['impliedVolatility']
                oi_row['PUTS_LTP'] = payload['records']['data'][m]['PE']['lastPrice']
                oi_row['PUTS_Net_Chng'] = payload['records']['data'][m]['PE']['change']
                if oi_mode == 'full':
                    oi_row['PUTS_Bid_Qty'] = payload['records']['data'][m]['PE']['buyQuantity1']
                    oi_row['PUTS_Bid_Price'] = payload['records']['data'][m]['PE']['buyPrice1']
                    oi_row['PUTS_Ask_Price'] = payload['records']['data'][m]['PE']['sellPrice1']
                    oi_row['PUTS_Ask_Qty'] = payload['records']['data'][m]['PE']['sellQuantity1']
            except KeyError:
                oi_row['PUTS_OI'], oi_row['PUTS_Chng_in_OI'], oi_row['PUTS_Volume'], oi_row['PUTS_IV'], oi_row[
                    'PUTS_LTP'], oi_row['PUTS_Net_Chng'] = 0, 0, 0, 0, 0, 0
                if oi_mode == 'full':
                    oi_row['PUTS_Bid_Qty'], oi_row['PUTS_Bid_Price'], oi_row['PUTS_Ask_Price'], oi_row[
                        'PUTS_Ask_Qty'] = 0, 0, 0, 0

            # if oi_mode == 'full':
            #     oi_row['CALLS_Chart'], oi_row['PUTS_Chart'] = 0, 0
            if oi_data.empty:
                oi_data = pd.DataFrame([oi_row]).copy()
            else:
                oi_data = pd.concat([oi_data, pd.DataFrame([oi_row])], ignore_index=True)
            oi_data['Symbol'] = symbol
            oi_data['Fetch_Time'] = payload['records']['timestamp']
    return oi_data


def fno_security_in_ban_period(trade_date: str):
    """
    To get the list of securities which are baned from fno segment to trade
    :param trade_date: eg:'20-06-2023'
    :return: pandas Data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    url = 'https://nsearchives.nseindia.com/archives/fo/sec_ban/fo_secban_'
    payload = f"{str(trade_date.strftime('%d%m%Y'))}.csv"
    request = nse_urlfetch(url + payload)
    securities = []
    if request.status_code == 200:
        lines = request.content.decode("utf-8").strip().split("\n")
        securities = [line.split(",")[1] for line in lines[1:]]
    elif request.status_code == 403:
        url2 = "https://www.nseindia.com/api/reports?archives=" \
             "%5B%7B%22name%22%3A%22F%26O%20-%20Security%20in%20ban%20period%22%2C%22type%22%3A%22archives%22%2C%22category%22" \
             f"%3A%22derivatives%22%2C%22section%22%3A%22equity%22%7D%5D&date={str(trade_date.strftime('%d-%b-%Y'))}" \
             f"&type=equity&mode=single"
        request = nse_urlfetch(url2)
        if request.status_code == 200:
            lines = request.content.decode("utf-8").strip().split("\n")
            securities = [line.split(",")[1] for line in lines[1:]]
        elif request.status_code == 403:
            raise FileNotFoundError(f' Data not found, change the date...')
    return securities


def live_most_active_underlying():
    """
    to get most active underlyings in live market, after market hour it will get as per last traded value
    link : https://www.nseindia.com/market-data/most-active-underlying
    :return: pandas dataframe
    """
    origin_url = "https://www.nseindia.com/market-data/most-active-underlying"
    url = f"https://www.nseindia.com/api/live-analysis-most-active-underlying"
    try:
        data_json = nse_urlfetch(url, origin_url=origin_url).json()
        data_df = pd.DataFrame(data_json['data'])
    except Exception as e:
        raise NSEdataNotFound(f" Resource not available MSG: {e}")
    return data_df


# if __name__ == '__main__':
    # df = future_price_volume_data("BANKNIFTY", "FUTIDX", from_date='01-11-2025', to_date='08-12-2025', period='6M')
    # df = option_price_volume_data('NIFTY', 'OPTIDX', period='1W')
    # df = nse_live_option_chain(symbol='TCS', expiry_date='02-01-2026')
    # df = fii_derivatives_statistics(trade_date='16-09-2024')
    # df = participant_wise_trading_volume(trade_date='16-09-2024')
    # df = fno_security_in_ban_period(trade_date='26-03-2025')
    # df = expiry_dates_option_index()
    # df = expiry_dates_future()
    # df = fno_bhav_copy('17-02-2025')
    # df = live_most_active_underlying()
    # print(df)
    # print(df.columns)
    # print(df[df['EXPIRY_DT']=='27-Jul-2023'])



class NiftySectoralIndices:
    indices_list = ["Nifty Auto", "Nifty Bank", "Nifty Chemicals", "Nifty Financial Services", "Nifty FMCG",
                    "Nifty Healthcare", "Nifty IT", "Nifty Media", "Nifty Metal", "Nifty Pharma", "Nifty Private Bank",
                    "Nifty PSU Bank", "Nifty Realty", "Nifty Consumer Durables", "Nifty Oil and Gas",
                    "Nifty500 Healthcare", "Nifty MidSmall Financial Services", "Nifty MidSmall Healthcare",
                    "Nifty MidSmall IT & Telecom"]
    index_constituent_list_urls = {
        "Nifty Auto": "https://www.niftyindices.com/IndexConstituent/ind_niftyautolist.csv",
        "Nifty Bank": "https://www.niftyindices.com/IndexConstituent/ind_niftybanklist.csv",
        "Nifty Chemicals": "https://niftyindices.com/IndexConstituent/ind_niftyChemicals_list.csv",
        "Nifty Financial Services": "https://www.niftyindices.com/IndexConstituent/ind_niftyfinancelist.csv",
        "Nifty FMCG": "https://www.niftyindices.com/IndexConstituent/ind_niftyfmcglist.csv",
        "Nifty Healthcare": "https://www.niftyindices.com/IndexConstituent/ind_niftyhealthcarelist.csv",
        "Nifty IT": "https://www.niftyindices.com/IndexConstituent/ind_niftyitlist.csv",
        "Nifty Media": "https://www.niftyindices.com/IndexConstituent/ind_niftymedialist.csv",
        "Nifty Metal": "https://www.niftyindices.com/IndexConstituent/ind_niftymetallist.csv",
        "Nifty Pharma": "https://www.niftyindices.com/IndexConstituent/ind_niftypharmalist.csv",
        "Nifty Private Bank": "https://www.niftyindices.com/IndexConstituent/ind_nifty_privatebanklist.csv",
        "Nifty PSU Bank": "https://www.niftyindices.com/IndexConstituent/ind_niftypsubanklist.csv",
        "Nifty Realty": "https://www.niftyindices.com/IndexConstituent/ind_niftyrealtylist.csv",
        "Nifty Consumer Durables": "https://www.niftyindices.com/IndexConstituent/ind_niftyconsumerdurableslist.csv",
        "Nifty Oil and Gas": "https://www.niftyindices.com/IndexConstituent/ind_niftyoilgaslist.csv",
        "Nifty500 Healthcare": "https://niftyindices.com/IndexConstituent/ind_nifty500Healthcare_list.csv",
        "Nifty MidSmall Financial Services": "https://www.niftyindices.com/IndexConstituent/ind_niftymidsmallfinancailservice_list.csv",
        "Nifty MidSmall Healthcare": "https://www.niftyindices.com/IndexConstituent/ind_niftymidsmallhealthcare_list.csv",
        "Nifty MidSmall IT & Telecom": "https://www.niftyindices.com/IndexConstituent/ind_niftymidsmallitAndtelecom_list.csv"
    }
    index_factsheet_urls = {
        "Nifty Auto": "https://www.niftyindices.com/Factsheet/ind_nifty_auto.pdf",
        "Nifty Bank": "https://www.niftyindices.com/Factsheet/ind_nifty_bank.pdf",
        "Nifty Chemicals": "https://niftyindices.com/Factsheet/Factsheet_Nifty_Chemicals.pdf",
        "Nifty Financial Services": "https://www.niftyindices.com/Factsheet/ind_Nifty_Financial_Services.pdf",
        "Nifty FMCG": "https://www.niftyindices.com/Factsheet/ind_nifty_FMCG.pdf",
        "Nifty Healthcare": "https://www.niftyindices.com/Factsheet/Factsheet_Nifty_Healthcare_Index.pdf",
        "Nifty IT": "https://www.niftyindices.com/Factsheet/ind_nifty_it.pdf",
        "Nifty Media": "https://www.niftyindices.com/Factsheet/ind_nifty_media.pdf",
        "Nifty Metal": "https://www.niftyindices.com/Factsheet/ind_nifty_metal.pdf",
        "Nifty Pharma": "https://www.niftyindices.com/Factsheet/ind_nifty_pharma.pdf",
        "Nifty Private Bank": "https://www.niftyindices.com/Factsheet/ind_nifty_private_bank.pdf",
        "Nifty PSU Bank": "https://www.niftyindices.com/Factsheet/ind_nifty_psu_bank.pdf",
        "Nifty Realty": "https://www.niftyindices.com/Factsheet/ind_nifty_realty.pdf",
        "Nifty Consumer Durables": "https://www.niftyindices.com/Factsheet/Factsheet_nifty_consumer_durables.pdf",
        "Nifty Oil and Gas": "https://www.niftyindices.com/Factsheet/Factsheet_nifty_oil_and_gas.pdf",
        "Nifty500 Healthcare": "https://niftyindices.com/Factsheet/Factsheet_Nifty500Healthcare.pdf",
        "Nifty MidSmall Financial Services": "https://www.niftyindices.com/Factsheet/Factsheet_NiftyMidSmallFinancialSevices.pdf",
        "Nifty MidSmall Healthcare": "https://www.niftyindices.com/Factsheet/Factsheet_NiftyMidSmallHealthCare.pdf",
        "Nifty MidSmall IT & Telecom": "https://www.niftyindices.com/Factsheet/Factsheet_NiftyMidSmallITAndTelecom.pdf"
    }


class NiftyBroadMarketIndices:
    indices_list = ["Nifty 50", "Nifty Next 50", "Nifty 100", "Nifty 200", "Nifty Total Market", "Nifty 500",
                    "Nifty Midcap150", "Nifty Midcap 50", "Nifty Midcap Select", "Nifty Midcap 100",
                    "Nifty Smallcap 250", "Nifty Smallcap 50", "Nifty Smallcap 100", "Nifty Microcap 250",
                    "Nifty LargeMidcap 250", "Nifty MidSmallcap 400", "Nifty India FPI 150", "India Vix"]
    index_constituent_list_urls = {
        "Nifty 50": "https://nsearchives.nseindia.com/content/indices/ind_nifty50list.csv",
        "Nifty Next 50": "https://nsearchives.nseindia.com/content/indices/ind_niftynext50list.csv",
        "Nifty 100": "https://nsearchives.nseindia.com/content/indices/ind_nifty100list.csv",
        "Nifty 200": "https://nsearchives.nseindia.com/content/indices/ind_nifty200list.csv",
        "Nifty Total Market": "https://nsearchives.nseindia.com/content/indices/ind_niftytotalmarket_list.csv",
        "Nifty 500": "https://nsearchives.nseindia.com/content/indices/ind_nifty500list.csv",
        "Nifty Midcap150": "https://nsearchives.nseindia.com/content/indices/ind_niftymidcap150list.csv",
        "Nifty Midcap 50": "https://nsearchives.nseindia.com/content/indices/ind_niftymidcap50list.csv",
        "Nifty Midcap Select": "https://nsearchives.nseindia.com/content/indices/ind_niftymidcapselect_list.csv",
        "Nifty Midcap 100": "https://nsearchives.nseindia.com/content/indices/ind_niftymidcap100list.csv",
        "Nifty Smallcap 250": "https://nsearchives.nseindia.com/content/indices/ind_niftysmallcap250list.csv",
        "Nifty Smallcap 50": "https://nsearchives.nseindia.com/content/indices/ind_niftysmallcap50list.csv",
        "Nifty Smallcap 100": "https://nsearchives.nseindia.com/content/indices/ind_niftysmallcap100list.csv",
        "Nifty Microcap 250": "https://nsearchives.nseindia.com/content/indices/ind_niftymicrocap250_list.csv",
        "Nifty LargeMidcap 250": "https://nsearchives.nseindia.com/content/indices/ind_niftylargemidcap250list.csv",
        "Nifty MidSmallcap 400": "https://nsearchives.nseindia.com/content/indices/ind_niftymidsmallcap400list.csv",
        "Nifty India FPI 150": "https://niftyindices.com/IndexConstituent/ind_niftyIndiaFPI150_list.csv"
    }
    index_factsheet_urls = {
        "Nifty 50": "https://nsearchives.nseindia.com/content/indices/ind_nifty50.pdf",
        "Nifty Next 50": "https://nsearchives.nseindia.com/content/indices/ind_next50.pdf",
        "Nifty 100": "https://nsearchives.nseindia.com/content/indices/ind_nifty_100.pdf",
        "Nifty 200": "https://nsearchives.nseindia.com/content/indices/ind_nifty_200.pdf",
        "Nifty Total Market": "https://nsearchives.nseindia.com/content/indices/Factsheet_NiftyTotalMarket.pdf",
        "Nifty 500": "https://nsearchives.nseindia.com/content/indices/ind_nifty_500.pdf",
        "Nifty Midcap150": "https://nsearchives.nseindia.com/content/indices/ind_Nifty_Midcap_150.pdf",
        "Nifty Midcap 50": "https://nsearchives.nseindia.com/content/indices/ind_nifty_midcap50.pdf",
        "Nifty Midcap Select": "https://nsearchives.nseindia.com/content/indices/Factsheet_NiftyMidcapSelect.pdf",
        "Nifty Midcap 100": "https://nsearchives.nseindia.com/content/indices/ind_niftymidcap100.pdf",
        "Nifty Smallcap 250": "https://nsearchives.nseindia.com/content/indices/ind_Nifty_Smallcap_250.pdf",
        "Nifty Smallcap 50": "https://nsearchives.nseindia.com/content/indices/ind_niftysmallcap_50.pdf",
        "Nifty Smallcap 100": "https://nsearchives.nseindia.com/content/indices/ind_niftysmallcap100.pdf",
        "Nifty Microcap 250": "https://nsearchives.nseindia.com/content/indices/Factsheet_Nifty_Microcap_250_Index.pdf",
        "Nifty LargeMidcap 250": "https://nsearchives.nseindia.com/content/indices/Factsheet_NIFTY_LargeMidcap_250_Index.pdf",
        "Nifty MidSmallcap 400": "https://nsearchives.nseindia.com/content/indices/ind_Nifty_MidSmallcap_400.pdf",
        "Nifty India FPI 150": "https://niftyindices.com/Factsheet/Factsheet_NiftyIndiaFPI150Index.pdf"
    }


class NiftyThematicIndices:
    indices_list = [
        "Nifty Capital Markets",
        "Nifty Commodities",
        "Nifty Conglomerate 50",
        "Nifty Core Housing",
        "Nifty CPSE",
        "Nifty EV & New Age Automotive",
        "Nifty Energy",
        "Nifty Housing",
        "Nifty India Consumption",
        "Nifty India Defence",
        "Nifty India Digital",
        "Nifty India Infrastructure & Logistics",
        "Nifty India Internet",
        "Nifty India Manufacturing",
        "Nifty India New Age Consumption",
        "Nifty India Railways PSU",
        "Nifty India Tourism",
        "Nifty Infrastructure",
        "Nifty IPO",
        "Nifty Midcap Liquid 15",
        "Nifty MNC",
        "Nifty Mobility",
        "Nifty PSE",
        "Nifty REITs & InvITs",
        "Nifty Rural",
        "Nifty Non-Cyclical Consumer",
        "Nifty Services Sector",
        "Nifty Shariah 25",
        "Nifty100 Liquid 15",
        "Nifty50 Shariah",
        "Nifty500 Shariah",
        "Nifty SME EMERGE",
        "Nifty Waves",
        "Nifty100 ESG",
        "Nifty100 Enhanced ESG",
        "Nifty100 ESG Sector Leaders",
        "Nifty India Corporate Group - Aditya Birla Group",
        "Nifty India Corporate Group - Mahindra Group",
        "Nifty India Corporate Group - Tata Group",
        "Nifty India Select 5 Corporate Groups (MAATR)"
    ]
    index_constituent_list_urls = {
        "Nifty Capital Markets": "https://niftyindices.com/IndexConstituent/ind_niftyCapitalMarkets_list.csv",
        "Nifty Commodities": "https://www.niftyindices.com/IndexConstituent/ind_niftycommoditieslist.csv",
        "Nifty Conglomerate 50": "https://www.niftyindices.com/IndexConstituent/ind_niftyConglomerate50_list.csv",
        "Nifty Core Housing": "https://niftyindices.com/IndexConstituent/ind_niftyCoreHousing_list.csv",
        "Nifty CPSE": "https://www.niftyindices.com/IndexConstituent/ind_niftycpselist.csv",
        "Nifty EV & New Age Automotive": "https://niftyindices.com/IndexConstituent/ind_niftyEv_NewAgeAutomotive_list.csv",
        "Nifty Energy": "https://www.niftyindices.com/IndexConstituent/ind_niftyenergylist.csv",
        "Nifty Housing": "https://www.niftyindices.com/IndexConstituent/ind_niftyhousing_list.csv",
        "Nifty India Consumption": "https://www.niftyindices.com/IndexConstituent/ind_niftyconsumptionlist.csv",
        "Nifty India Defence": "https://www.niftyindices.com/IndexConstituent/ind_niftyindiadefence_list.csv",
        "Nifty India Digital": "https://niftyindices.com/IndexConstituent/ind_niftyindiadigital_list.csv",
        "Nifty India Infrastructure & Logistics": "https://niftyindices.com/IndexConstituent/ind_niftyIndiaInfrastructure_Logistics_list.csv",
        "Nifty India Internet": "https://www.niftyindices.com/IndexConstituent/ind_niftyIndiaInternet_list.csv",
        "Nifty India Manufacturing": "https://www.niftyindices.com/IndexConstituent/ind_niftyindiamanufacturing_list.csv",
        "Nifty India New Age Consumption": "https://niftyindices.com/IndexConstituent/ind_niftyIndiaNewAgeConsumption_list.csv",
        "Nifty India Railways PSU": "https://niftyindices.com/IndexConstituent/ind_niftyIndiaRailwaysPSU_list.csv",
        "Nifty India Tourism": "https://niftyindices.com/IndexConstituent/ind_niftyindiatourism_list.csv",
        "Nifty Infrastructure": "https://www.niftyindices.com/IndexConstituent/ind_niftyinfralist.csv",
        "Nifty IPO": "https://niftyindices.com/IndexConstituent/ind_niftyIPO_list.csv",
        "Nifty Midcap Liquid 15": "https://www.niftyindices.com/IndexConstituent/ind_Nifty_Midcap_Liquid15.csv",
        "Nifty MNC": "https://www.niftyindices.com/IndexConstituent/ind_niftymnclist.csv",
        "Nifty Mobility": "https://www.niftyindices.com/IndexConstituent/ind_niftymobility_list.csv",
        "Nifty PSE": "https://www.niftyindices.com/IndexConstituent/ind_niftypselist.csv",
        "Nifty REITs & InvITs": "https://www.niftyindices.com/IndexConstituent/ind_niftyREITs_InvITs_list.csv",
        "Nifty Rural": "https://niftyindices.com/IndexConstituent/ind_niftyRural_list.csv",
        "Nifty Non-Cyclical Consumer": "https://www.niftyindices.com/IndexConstituent/ind_niftynon-cyclicalconsumer_list.csv",
        "Nifty Services Sector": "https://www.niftyindices.com/IndexConstituent/ind_niftyservicelist.csv",
        "Nifty Shariah 25": "",
        "Nifty100 Liquid 15": "https://www.niftyindices.com/IndexConstituent/ind_Nifty100_Liquid15.csv",
        "Nifty50 Shariah": "",
        "Nifty500 Shariah": "",
        "Nifty SME EMERGE": "https://www.niftyindices.com/IndexConstituent/ind_niftysmelist.csv",
        "Nifty Waves": "https://niftyindices.com/IndexConstituent/ind_niftyWaves_list.csv",
        "Nifty100 ESG": "",
        "Nifty100 Enhanced ESG": "",
        "Nifty100 ESG Sector Leaders": "",
        "Nifty India Corporate Group - Aditya Birla Group": "https://niftyindices.com/IndexConstituent/ind_nifty_adityabirlalist.csv",
        "Nifty India Corporate Group - Mahindra Group": "https://niftyindices.com/IndexConstituent/ind_nifty_mahindralist.csv",
        "Nifty India Corporate Group - Tata Group": "https://niftyindices.com/IndexConstituent/ind_nifty_tatalist.csv",
        "Nifty India Select 5 Corporate Groups (MAATR)": "https://niftyindices.com/IndexConstituent/ind_niftyIndiaSelect5CorporateGroupsMAATR_list.csv"
    }
    index_factsheet_urls = {
        "Nifty Capital Markets": "https://niftyindices.com/Factsheet/Factsheet_NiftyCapitalMarkets.pdf",
        "Nifty Commodities": "https://www.niftyindices.com/Factsheet/ind_nifty_commodities.pdf",
        "Nifty Conglomerate 50": "https://www.niftyindices.com/Factsheet/Factsheet_NiftyConglomerate50.pdf",
        "Nifty Core Housing": "https://niftyindices.com/Factsheet/Nifty_Core_Housing_Factsheet.pdf",
        "Nifty CPSE": "https://www.niftyindices.com/Factsheet/ind_Nifty_CPSE.pdf",
        "Nifty EV & New Age Automotive": "https://niftyindices.com/Factsheet/Factsheet_NiftyEVNewAgeAutomotive.pdf",
        "Nifty Energy": "https://www.niftyindices.com/Factsheet/ind_nifty_energy.pdf",
        "Nifty Housing": "https://www.niftyindices.com/Factsheet/Factsheet_NiftyHousing.pdf",
        "Nifty India Consumption": "https://www.niftyindices.com/Factsheet/ind_Nifty_India_Consumption.pdf",
        "Nifty India Defence": "https://www.niftyindices.com/Factsheet/Factsheet_NiftyIndiaDefence.pdf",
        "Nifty India Digital": "https://niftyindices.com/Factsheet/Factsheet_NiftyIndiaDigital.pdf",
        "Nifty India Infrastructure & Logistics": "https://niftyindices.com/Factsheet/Factsheet_Nifty_India_Infrastructure_and_Logistics.pdf",
        "Nifty India Internet": "https://www.niftyindices.com/Factsheet/Factsheet_Nifty_India_Internet.pdf",
        "Nifty India Manufacturing": "https://www.niftyindices.com/Factsheet/Factsheet_Nifty_India_Manufacturing_Index.pdf",
        "Nifty India New Age Consumption": "https://niftyindices.com/Factsheet/Factsheet_NiftyIndiaNewAgeConsumption.pdf",
        "Nifty India Railways PSU": "https://niftyindices.com/Factsheet/Factsheet_NiftyIndiaRailwaysPSU.pdf",
        "Nifty India Tourism": "https://niftyindices.com/Factsheet/Factsheet_NiftyIndiaTourism.pdf",
        "Nifty Infrastructure": "https://www.niftyindices.com/Factsheet/ind_nifty_infra.pdf",
        "Nifty IPO": "https://niftyindices.com/Factsheet/Factsheet_NiftyIPO.pdf",
        "Nifty Midcap Liquid 15": "https://www.niftyindices.com/Factsheet/Nifty_midcap_liquid_15.pdf",
        "Nifty MNC": "https://www.niftyindices.com/Factsheet/ind_nifty_mnc.pdf",
        "Nifty Mobility": "https://www.niftyindices.com/Factsheet/Factsheet_Nifty_Mobility_Total_Returns_Index.pdf",
        "Nifty PSE": "https://www.niftyindices.com/Factsheet/ind_nifty_pse.pdf",
        "Nifty REITs & InvITs": "https://www.niftyindices.com/Factsheet/Factsheet_REITs_InvITs.pdf",
        "Nifty Rural": "https://niftyindices.com/Factsheet/Factsheet_NiftyRural.pdf",
        "Nifty Non-Cyclical Consumer": "https://www.niftyindices.com/Factsheet/Factsheet_NiftyNonCyclicalConsumer.pdf",
        "Nifty Services Sector": "https://www.niftyindices.com/Factsheet/ind_nifty_services_sector.pdf",
        "Nifty Shariah 25": "https://www.niftyindices.com/Factsheet/Factsheet_nifty_Shariah25.pdf",
        "Nifty100 Liquid 15": "https://www.niftyindices.com/Factsheet/Nifty100_Liquid_15.pdf",
        "Nifty50 Shariah": "https://www.niftyindices.com/Factsheet/Factsheet_Nifty50_Shariah.pdf",
        "Nifty500 Shariah": "https://www.niftyindices.com/Factsheet/Factsheet_Nifty500_Shariah.pdf",
        "Nifty SME EMERGE": "https://www.niftyindices.com/Factsheet/Factsheet_NIFTY_SME_EMERGE_Index.pdf",
        "Nifty Waves": "https://niftyindices.com/Factsheet/Factsheet_Nifty_Waves.pdf",
        "Nifty100 ESG": "https://www.niftyindices.com/Factsheet/Factsheet_NIFTY100_ESG_Index.pdf",
        "Nifty100 Enhanced ESG": "https://www.niftyindices.com/Factsheet/Factsheet_NIFTY100_Enhanced_ESG_Index.pdf",
        "Nifty100 ESG Sector Leaders": "https://www.niftyindices.com/Factsheet/Factsheet_Nifty100_ESG_Sector_Leaders.pdf",
        "Nifty India Corporate Group - Aditya Birla Group": "https://niftyindices.com/Factsheet/ind_NIFTY_ADITYA_BIRLA_GROUP.pdf",
        "Nifty India Corporate Group - Mahindra Group": "https://niftyindices.com/Factsheet/ind_NIFTY_MAHINDRA_GROUP.pdf",
        "Nifty India Corporate Group - Tata Group": "https://niftyindices.com/Factsheet/Ind_NIFTY_TATA_GROUP.pdf",
        "Nifty India Select 5 Corporate Groups (MAATR)": "https://niftyindices.com/Factsheet/Factsheet_NiftyIndiaSelect5CorporateGroupsMAATR.pdf"
    }


class NiftyStrategyIndices:
    indices_list = [
        "Nifty100 Equal Weight",
        "Nifty100 Low Volatility 30",
        "Nifty200 Momentum 30",
        "Nifty200 Alpha 30",
        "Nifty100 Alpha 30",
        "Nifty Dividend Opportunities 50",
        "Nifty Growth Sectors 15",
        "Nifty High Beta 50",
        "Nifty Low Volatility 50",
        "Nifty Top 10 Equal Weight",
        "Nifty Top 15 Equal Weight",
        "Nifty Top 20 Equal Weight",
        "Nifty100 Quality 30",
        "Nifty Midcap150 Momentum 50",
        "Nifty Midcap150 Quality 50",
        "Nifty Alpha 50",
        "Nifty50 Equal Weight",
        "Nifty50 Value 20",
        "Nifty200 Value 30",
        "Nifty500 Flexicap Quality 30",
        "Nifty500 Value 50",
        "Nifty500 Equal Weight",
        "Nifty500 Low Volatility 50",
        "Nifty500 Momentum 50",
        "Nifty500 Quality 50",
        "Nifty500 Multifactor MQVLv 50",
        "Nifty200 Quality 30",
        "Nifty MidSmallcap400 Momentum Quality 100",
        "Nifty Smallcap250 Quality 50",
        "Nifty Total Market Momentum Quality 50"
    ]
    index_constituent_list_urls = {
        "Nifty100 Equal Weight": "https://www.niftyindices.com/IndexConstituent/ind_nifty100list.csv",
        "Nifty100 Low Volatility 30": "https://www.niftyindices.com/IndexConstituent/ind_Nifty100LowVolatility30list.csv",
        "Nifty200 Momentum 30": "https://www.niftyindices.com/IndexConstituent/ind_nifty200Momentum30_list.csv",
        "Nifty200 Alpha 30": "https://www.niftyindices.com/IndexConstituent/ind_nifty200alpha30_list.csv",
        "Nifty100 Alpha 30": "https://www.niftyindices.com/IndexConstituent/ind_nifty100Alpha30list.csv",
        "Nifty Dividend Opportunities 50": "https://www.niftyindices.com/IndexConstituent/ind_niftydivopp50list.csv",
        "Nifty Growth Sectors 15": "https://www.niftyindices.com/IndexConstituent/ind_NiftyGrowth_Sectors15_Index.csv",
        "Nifty High Beta 50": "https://www.niftyindices.com/IndexConstituent/nifty_High_Beta50_Index.csv",
        "Nifty Low Volatility 50": "https://www.niftyindices.com/IndexConstituent/nifty_low_Volatility50_Index.csv",
        "Nifty Top 10 Equal Weight": "https://niftyindices.com/IndexConstituent/ind_niftytop10EqualWeight_list.csv",
        "Nifty Top 15 Equal Weight": "https://niftyindices.com/IndexConstituent/ind_niftytop15EqualWeight_list.csv",
        "Nifty Top 20 Equal Weight": "https://niftyindices.com/IndexConstituent/ind_niftyTop20EqualWeight_list.csv",
        "Nifty100 Quality 30": "https://www.niftyindices.com/IndexConstituent/ind_nifty100Quality30list.csv",
        "Nifty Midcap150 Momentum 50": "https://www.niftyindices.com/IndexConstituent/ind_niftymidcap150momentum50_list.csv",
        "Nifty Midcap150 Quality 50": "https://www.niftyindices.com/IndexConstituent/ind_niftymidcap150quality50list.csv",
        "Nifty Alpha 50": "https://www.niftyindices.com/IndexConstituent/ind_nifty_Alpha_Index.csv",
        "Nifty50 Equal Weight": "https://www.niftyindices.com/IndexConstituent/ind_Nifty50EqualWeight.csv",
        "Nifty50 Value 20": "https://www.niftyindices.com/IndexConstituent/ind_Nifty50_Value20.csv",
        "Nifty200 Value 30": "https://niftyindices.com/IndexConstituent/ind_nifty200Value30_list.csv",
        "Nifty500 Flexicap Quality 30": "https://niftyindices.com/IndexConstituent/ind_nifty500FlexicapQuality30_list.csv",
        "Nifty500 Value 50": "https://www.niftyindices.com/IndexConstituent/ind_nifty500Value50_list.csv",
        "Nifty500 Equal Weight": "https://niftyindices.com/IndexConstituent/ind_nifty500EqualWeight_list.csv",
        "Nifty500 Low Volatility 50": "https://niftyindices.com/IndexConstituent/ind_nifty500LowVolatility50_list.csv",
        "Nifty500 Momentum 50": "https://www.niftyindices.com/IndexConstituent/ind_nifty500Momentum50_list.csv",
        "Nifty500 Quality 50": "https://niftyindices.com/IndexConstituent/ind_nifty500Quality50_list.csv",
        "Nifty500 Multifactor MQVLv 50": "https://niftyindices.com/IndexConstituent/ind_nifty500MultifactorMQVLv50_list.csv",
        "Nifty200 Quality 30": "https://www.niftyindices.com/IndexConstituent/ind_nifty200Quality30_list.csv",
        "Nifty MidSmallcap400 Momentum Quality 100": "https://www.niftyindices.com/IndexConstituent/ind_niftyMidSmallcap400MomentumQuality100_list.csv",
        "Nifty Smallcap250 Quality 50": "https://www.niftyindices.com/IndexConstituent/ind_niftySmallcap250_Quality50_list.csv",
        "Nifty Total Market Momentum Quality 50": "https://niftyindices.com/IndexConstituent/ind_niftyTotalMarketMomentumQuality50_list.csv"
    }
    index_factsheet_urls = {
        "Nifty100 Equal Weight": "https://www.niftyindices.com/Factsheet/Factsheet_Nifty_100_EW_Index.pdf",
        "Nifty100 Low Volatility 30": "https://www.niftyindices.com/Factsheet/Nifty100_LowVolatility30.pdf",
        "Nifty200 Momentum 30": "https://www.niftyindices.com/Factsheet/Factsheet_Nifty200_Momentum30.pdf",
        "Nifty200 Alpha 30": "https://www.niftyindices.com/Factsheet/Factsheet_Nifty200Alpha30.pdf",
        "Nifty100 Alpha 30": "https://www.niftyindices.com/Factsheet/Factsheet_Nifty100_Alpha30.pdf",
        "Nifty Dividend Opportunities 50": "https://www.niftyindices.com/Factsheet/ind_Nifty_Divid_Opp50.pdf",
        "Nifty Growth Sectors 15": "https://www.niftyindices.com/Factsheet/Nifty_Growth_Sectors15.pdf",
        "Nifty High Beta 50": "https://www.niftyindices.com/Factsheet/Factsheet_nifty_High_Beta50.pdf",
        "Nifty Low Volatility 50": "https://www.niftyindices.com/Factsheet/Factsheet_nifty_Low_Volatility50.pdf",
        "Nifty Top 10 Equal Weight": "https://niftyindices.com/Factsheet/Factsheet_NiftyTop10EqualWeight.pdf",
        "Nifty Top 15 Equal Weight": "https://niftyindices.com/Factsheet/Factsheet_NiftyTop15EqualWeight.pdf",
        "Nifty Top 20 Equal Weight": "https://niftyindices.com/Factsheet/Factsheet_NiftyTop20EqualWeight.pdf",
        "Nifty100 Quality 30": "https://www.niftyindices.com/Factsheet/Nifty100_Quality30.pdf",
        "Nifty Midcap150 Momentum 50": "https://www.niftyindices.com/Factsheet/Factsheet_NiftyMidcap150Momentum50.pdf",
        "Nifty Midcap150 Quality 50": "https://www.niftyindices.com/Factsheet/Factsheet_NIFTYMidcap150_Quality50.pdf",
        "Nifty Alpha 50": "https://www.niftyindices.com/Factsheet/Factsheet_Nifty_Alpha50.pdf",
        "Nifty50 Equal Weight": "https://www.niftyindices.com/Factsheet/Factsheet_NIFTY50_Equal_Weight.pdf",
        "Nifty50 Value 20": "https://www.niftyindices.com/Factsheet/Nifty50_Value20.pdf",
        "Nifty200 Value 30": "https://niftyindices.com/Factsheet/Factsheet_Nifty200_Value30.pdf",
        "Nifty500 Flexicap Quality 30": "https://niftyindices.com/Factsheet/Factsheet_Nifty500_Flexicap_Quality_30.pdf",
        "Nifty500 Value 50": "https://www.niftyindices.com/Factsheet/FactsheetNIFTY500Value50.pdf",
        "Nifty500 Equal Weight": "https://niftyindices.com/Factsheet/Factsheet_Nifty_500_Equal_Weight.pdf",
        "Nifty500 Low Volatility 50": "https://niftyindices.com/Factsheet/Factsheet_Nifty500LowVolatility50.pdf",
        "Nifty500 Momentum 50": "https://www.niftyindices.com/Factsheet/FactsheetNifty500Momentum50.pdf",
        "Nifty500 Quality 50": "https://niftyindices.com/Factsheet/Factsheet_Nifty500Quality50.pdf",
        "Nifty500 Multifactor MQVLv 50": "https://niftyindices.com/Factsheet/Factsheet_Nifty500MultifactorMQVLv50.pdf",
        "Nifty200 Quality 30": "https://www.niftyindices.com/Factsheet/Factsheet_NIFTY200_Quality30.pdf",
        "Nifty MidSmallcap400 Momentum Quality 100": "https://www.niftyindices.com/Factsheet/Factsheet_NiftyMidSmallcap400MomentumQuality100.pdf",
        "Nifty Smallcap250 Quality 50": "https://www.niftyindices.com/Factsheet/Factsheet_NiftySmallcap250Quality50.pdf",
        "Nifty Total Market Momentum Quality 50": "https://niftyindices.com/Factsheet/Factsheet_NiftyTotalMarketMomentumQuality50.pdf"
    }



from .capital_market_data import price_volume_and_deliverable_position_data, price_volume_data, \
    deliverable_position_data, bulk_deal_data, block_deals_data, short_selling_data, \
    bhav_copy_with_delivery, bhav_copy_equities, equity_list, fno_equity_list, fno_index_list, nifty50_equity_list, \
    niftynext50_equity_list, niftymidcap150_equity_list, niftysmallcap250_equity_list, india_vix_data, \
    index_data, market_watch_all_indices, var_begin_day, var_1st_intra_day, var_2nd_intra_day, var_3rd_intra_day, \
    var_4th_intra_day, var_end_of_day, sme_bhav_copy, sme_band_complete, week_52_high_low_report, financial_results_for_equity, \
    corporate_bond_trade_report, bhav_copy_sme, pe_ratio, corporate_actions_for_equity, event_calendar_for_equity, \
    top_gainers_or_losers, most_active_equities, total_traded_stocks



from io import BytesIO
from nselib.libutil import *
import config as conf
import pandas as pd


def get_class(index_category: str):
    category = f"Nifty{index_category}"
    category_class = getattr(conf, category, None)

    if category_class is None:
        raise ValueError(f"No such category in config: {category}")
    return category_class


def index_list(index_category: str = 'BroadMarketIndices'):
    """
     to get the available NSE indices for each category of indices. there ar 4 category defined by NSE link as below
     https://www.nseindia.com/static/products-services/about-indices..
    :param index_category: SectoralIndices/ BroadMarketIndices/ ThematicIndices/ StrategyIndices
    :return: list
    """
    return get_class(index_category).indices_list


def validate_index_category(index_category: str = 'BroadMarketIndices'):
    category_list = ['SectoralIndices', 'BroadMarketIndices', 'ThematicIndices', 'StrategyIndices']
    if index_category in category_list:
        pass
    else:
        raise ValueError(f'{index_category} is not a valid index_category:: please select category_list from list :  {category_list}')


def validate_index_name(index_category: str = 'BroadMarketIndices', index_name: str = 'Nifty 50'):
    validate_index_category(index_category)
    ind_list = index_list(index_category)
    if index_name in ind_list:
        pass
    else:
        raise ValueError(f'{index_name} is not a valid index_name:: please select index name from list :  {ind_list}')


def constituent_stock_list(index_category: str = 'BroadMarketIndices', index_name: str = 'Nifty 50'):
    """
    to get list of all that stocks constituent with the given index and index_category.
    :param index_category: SectoralIndices/ BroadMarketIndices/ ThematicIndices/ StrategyIndices
    :param index_name: select name of index from the index list provided by get_index_list
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_index_name(index_category, index_name)
    url = get_class(index_category).index_constituent_list_urls[index_name]
    if not url:
        raise FileNotFoundError(f' Data not found for index {index_name}')
    response = nse_urlfetch(url)
    if response.status_code == 200:
        stocks_df = pd.read_csv(BytesIO(response.content))
    else:
        raise FileNotFoundError(f' Data not found, check index_name or index_category ...')
    url_fs = get_class(index_category).index_factsheet_urls[index_name]
    print(f" Note: For more detail information related to {index_name} Please check the Fact Sheet - {url_fs}")
    return stocks_df


def live_index_performances():
    """
    to get index performances in live market, after market hour it will get as per last traded value
    link : https://www.nseindia.com/market-data/index-performances
    :return:
    """
    origin_url = "https://www.nseindia.com/market-data/index-performances"
    url = f"https://www.nseindia.com/api/allIndices"
    try:
        data_json = nse_urlfetch(url, origin_url=origin_url).json()
        data_df = pd.DataFrame(data_json['data'])
    except Exception as e:
        raise NSEdataNotFound(f" Resource not available MSG: {e}")
    data_df.drop(columns=['chartTodayPath', 'chart30dPath', 'chart365dPath'], inplace=True)
    return data_df


# if __name__ == '__main__':

    # data = constituent_stock_list(index_category='BroadMarketIndices', index_name='Nifty  50')
    # data = live_index_performances()
    # print(data)



from .derivative_data import future_price_volume_data, option_price_volume_data, fno_bhav_copy, \
    participant_wise_open_interest, participant_wise_trading_volume, expiry_dates_future, expiry_dates_option_index,\
    nse_live_option_chain, fii_derivatives_statistics, fno_security_in_ban_period, live_most_active_underlying



import logging
import sys


# Create a custom logger
def mylogger(logger):
    logger.setLevel(logging.DEBUG)
    # Create handlers
    c_handler = logging.StreamHandler(sys.stdout)
    # Create formatters and add it to handlers
    f_format = logging.Formatter('%(name)s::%(levelname)s::%(message)s')
    c_handler.setFormatter(f_format)
    # Add handlers to the logger
    logger.addHandler(c_handler)
    return logger



from io import BytesIO, StringIO
import json
from nselib.libutil import *
from nselib.constants import *


def get_future_price_volume_data(symbol: str, instrument: str, from_date: str, to_date: str):
    origin_url = "https://www.nseindia.com/report-detail/fo_eq_security"
    url = "https://www.nseindia.com/api/historicalOR/foCPV?"
    payload = f"from={from_date}&to={to_date}&instrumentType={instrument}&symbol={symbol}&csv=true"
    try:
        data_dict = nse_urlfetch(url + payload, origin_url=origin_url).json()
    except Exception as e:
        raise ValueError(f" Invalid parameters : NSE error:{e}")
    data_df = pd.DataFrame(data_dict['data'])
    data_df.columns = cleaning_column_name(data_df.columns)
    return data_df


def get_option_price_volume_data(symbol: str, instrument: str, option_type: str, from_date: str, to_date: str):
    origin_url = "https://www.nseindia.com/report-detail/fo_eq_security"
    url = "https://www.nseindia.com/api/historicalOR/foCPV?"
    payload = f"from={from_date}&to={to_date}&instrumentType={instrument}&symbol={symbol}" \
              f"&optionType={option_type}&csv=true"
    try:
        data_dict = nse_urlfetch(url + payload, origin_url=origin_url).json()
    except Exception as e:
        raise ValueError(f" Invalid parameters : NSE error : {e}")
    data_df = pd.DataFrame(data_dict['data'])
    if data_df.empty:
        raise ValueError(f"Invalid parameters, Please change the parameters")
    data_df.columns = cleaning_column_name(data_df.columns)
    return data_df[future_price_volume_data_column]


def get_nse_option_chain(symbol: str, expiry_date: str):
    """
    get NSE option chain for the symbol
    :param expiry_date: in 'DD-MMM-YYYY' format eg='25-Dec-2025'
    :param symbol: eg:'TCS'/'BANKNIFTY'
    :return: pandas dataframe
    """
    symbol = cleaning_nse_symbol(symbol)
    origin_url = "https://www.nseindia.com/option-chain"

    if any(x in symbol for x in indices_list):
        url = f'https://www.nseindia.com/api/option-chain-v3?type=Indices&symbol={symbol}&expiry={expiry_date}'
    else:
        url = f'https://www.nseindia.com/api/option-chain-v3?type=Equity&symbol={symbol}&expiry={expiry_date}'
    payload = nse_urlfetch(url, origin_url=origin_url)
    return payload






import datetime as dt
import zipfile
import xml.etree.ElementTree as ET
from nselib.capital_market.get_func import *


# logging.basicConfig(level=logging.DEBUG)
# logging.getLogger("urllib3").setLevel(logging.WARNING)


def price_volume_and_deliverable_position_data(symbol: str, from_date: str = None, to_date: str = None,
                                               period: str = None):
    """
    get Security wise price volume & Deliverable position data set. use get_nse_symbols() to get all symbols
    :param symbol: symbol eg: 'SBIN'
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,'1W': for last 7 days data,
                            '1M': from last month same date, '6M': last 6 month data, '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    symbol = cleaning_nse_symbol(symbol=symbol)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=price_volume_and_deliverable_position_data_columns)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 365:
            end_date = (from_date + dt.timedelta(364)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_price_volume_and_deliverable_position_data(symbol=symbol, from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(365)
        load_days = (to_date - from_date).days
        if not data_df.empty:
            # Drop all-NaN columns from both frames
            data_df = data_df.fillna('-')
            nse_df = nse_df.fillna('-')
            data_df = data_df.dropna(axis=1, how='all')
            nse_df = nse_df.dropna(axis=1, how='all')
            if not data_df.empty:
                nse_df = pd.concat([nse_df, data_df], ignore_index=True)

    nse_df["TotalTradedQuantity"] = pd.to_numeric(nse_df["TotalTradedQuantity"].astype(str).str.replace(",", ""), errors="coerce")
    nse_df["TurnoverInRs"] = pd.to_numeric(nse_df["TurnoverInRs"].astype(str).str.replace(",", ""), errors="coerce")
    nse_df["No.ofTrades"] = pd.to_numeric(nse_df["No.ofTrades"].astype(str).str.replace(",", ""), errors="coerce")
    nse_df["DeliverableQty"] = pd.to_numeric(nse_df["DeliverableQty"].astype(str).str.replace(",", ""), errors="coerce")
    return nse_df


def price_volume_data(symbol: str, from_date: str = None, to_date: str = None, period: str = None):
    """
    get Security wise price volume data set.
    :param symbol: symbol eg: 'SBIN'
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,'1W': for last 7 days data,
                            '1M': from last month same date, '6M': last 6 month data, '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    symbol = cleaning_nse_symbol(symbol=symbol)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=price_volume_data_columns)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 365:
            end_date = (from_date + dt.timedelta(364)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_price_volume_data(symbol=symbol, from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(365)
        load_days = (to_date - from_date).days
        if nse_df.empty:
            nse_df = data_df
        else:
            nse_df = pd.concat([nse_df, data_df], ignore_index=True)
    return nse_df


def deliverable_position_data(symbol: str, from_date: str = None, to_date: str = None, period: str = None):
    """
    get Security wise deliverable position data set.
    :param symbol: symbol eg: 'SBIN'
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,
                            '1W': for last 7 days data,
                            '1M': from last month same date,
                            '6M': last 6 month data,
                            '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    symbol = cleaning_nse_symbol(symbol=symbol)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=deliverable_data_columns)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 365:
            end_date = (from_date + dt.timedelta(364)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_deliverable_position_data(symbol=symbol, from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(365)
        load_days = (to_date - from_date).days
        if nse_df.empty:
            nse_df = data_df
        else:
            nse_df = pd.concat([nse_df, data_df], ignore_index=True)
    return nse_df


def india_vix_data(from_date: str = None, to_date: str = None, period: str = None):
    """
    get india vix spot data  set for the specific time period.
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,'1W': for last 7 days data,
                            '1M': from last month same date, '6M': last 6 month data, '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=india_vix_data_column)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 365:
            end_date = (from_date + dt.timedelta(364)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_india_vix_data(from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(365)
        load_days = (to_date - from_date).days
        if nse_df.empty:
            nse_df = data_df
        else:
            nse_df = pd.concat([nse_df, data_df], ignore_index=True)
    return nse_df


def index_data(index: str, from_date: str = None, to_date: str = None, period: str = None):
    """
    get historical index data set for the specific time period.
    apply the index name as per the nse india site
    :param index: 'NIFTY 50'/'NIFTY BANK'
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,'1W': for last 7 days data,
                            '1M': from last month same date, '6M': last 6 month data, '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame()
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 365:
            end_date = (from_date + dt.timedelta(364)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_index_data(index=index, from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(365)
        load_days = (to_date - from_date).days
        if nse_df.empty:
            nse_df = data_df
        else:
            nse_df = pd.concat([nse_df, data_df], ignore_index=True)
    return nse_df


def bulk_deal_data(from_date: str = None, to_date: str = None, period: str = None):
    """
    get bulk deal data set.
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,
                            '1W': for last 7 days data,
                            '1M': from last month same date,
                            '6M': last 6 month data,
                            '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=bulk_deal_data_columns)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 365:
            end_date = (from_date + dt.timedelta(364)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_bulk_deal_data(from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(365)
        load_days = (to_date - from_date).days
        if nse_df.empty:
            nse_df = data_df
        else:
            nse_df = pd.concat([nse_df, data_df], ignore_index=True)
    return nse_df


def block_deals_data(from_date: str = None, to_date: str = None, period: str = None):
    """
    get block deals data set.
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,
                            '1W': for last 7 days data,
                            '1M': from last month same date,
                            '6M': last 6 month data,
                            '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=block_deals_data_columns)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 365:
            end_date = (from_date + dt.timedelta(364)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_block_deals_data(from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(365)
        load_days = (to_date - from_date).days
        if nse_df.empty:
            nse_df = data_df
        else:
            nse_df = pd.concat([nse_df, data_df], ignore_index=True)
    return nse_df


def short_selling_data(from_date: str = None, to_date: str = None, period: str = None):
    """
    get short selling data set.
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,
                            '1W': for last 7 days data,
                            '1M': from last month same date,
                            '6M': last 6 month data,
                            '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    nse_df = pd.DataFrame(columns=short_selling_data_columns)
    from_date = datetime.strptime(from_date, dd_mm_yyyy)
    to_date = datetime.strptime(to_date, dd_mm_yyyy)
    load_days = (to_date - from_date).days
    while load_days > 0:
        if load_days > 365:
            end_date = (from_date + dt.timedelta(364)).strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        else:
            end_date = to_date.strftime(dd_mm_yyyy)
            start_date = from_date.strftime(dd_mm_yyyy)
        data_df = get_short_selling_data(from_date=start_date, to_date=end_date)
        from_date = from_date + dt.timedelta(365)
        load_days = (to_date - from_date).days
        if nse_df.empty:
            nse_df = data_df
        else:
            nse_df = pd.concat([nse_df, data_df], ignore_index=True)
    return nse_df


def bhav_copy_with_delivery(trade_date: str):
    """
    get the NSE bhav copy with delivery data as per the traded date
    :param trade_date: eg:'20-06-2023'
    :return: pandas data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    use_date = trade_date.strftime(ddmmyyyy)
    url = f'https://nsearchives.nseindia.com/products/content/sec_bhavdata_full_{use_date}.csv'
    request_bhav = nse_urlfetch(url)
    if request_bhav.status_code == 200:
        bhav_df = pd.read_csv(BytesIO(request_bhav.content))
    else:
        raise FileNotFoundError(f' Data not found, change the trade_date...')
    bhav_df.columns = [name.replace(' ', '') for name in bhav_df.columns]
    bhav_df['SERIES'] = bhav_df['SERIES'].str.replace(' ', '')
    bhav_df['DATE1'] = bhav_df['DATE1'].str.replace(' ', '')
    return bhav_df


def bhav_copy_equities(trade_date: str):
    """
    get new CM-UDiFF Common Bhavcopy Final as per the traded date provided
    :param trade_date:
    :return: pandas dataframe
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    url = 'https://nsearchives.nseindia.com/content/cm/BhavCopy_NSE_CM_0_0_0_'
    payload = f"{str(trade_date.strftime('%Y%m%d'))}_F_0000.csv.zip"
    request_bhav = nse_urlfetch(url + payload)
    bhav_df = pd.DataFrame()
    if request_bhav.status_code == 200:
        zip_bhav = zipfile.ZipFile(BytesIO(request_bhav.content), 'r')
        for file_name in zip_bhav.filelist:
            if file_name:
                bhav_df = pd.read_csv(zip_bhav.open(file_name))
    elif request_bhav.status_code == 403:
        raise FileNotFoundError(f' Data not found, change the trade_date...')
    # bhav_df = bhav_df[['SYMBOL', 'SERIES', 'OPEN', 'HIGH', 'LOW', 'CLOSE', 'LAST', 'PREVCLOSE', 'TOTTRDQTY',
    #                    'TOTTRDVAL', 'TIMESTAMP', 'TOTALTRADES']]
    return bhav_df


def bhav_copy_indices(trade_date: str):
    """
    get nse bhav copy as per the traded date provided
    :param trade_date: eg:'20-06-2023'
    :return: pandas dataframe
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    url = f"https://nsearchives.nseindia.com/content/indices/ind_close_all_{str(trade_date.strftime('%d%m%Y').upper())}.csv"
    file_chk = nse_urlfetch(url)
    if file_chk.status_code != 200:
        raise FileNotFoundError(f" No data available for : {trade_date}")
    try:
        bhav_df = pd.read_csv(BytesIO(file_chk.content))
    except Exception as e:
        raise FileNotFoundError(f' Bhav copy indices not found for : {trade_date} :: NSE error : {e}')
    return bhav_df


def bhav_copy_sme(trade_date: str):
    """
    get the NSE bhav copy for SME data as per the traded date
    :param trade_date: eg:'20-06-2023'
    :return: pandas data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    use_date = trade_date.strftime(ddmmyy)
    url = f'https://nsearchives.nseindia.com/archives/sme/bhavcopy/sme{use_date}.csv'
    request_bhav = nse_urlfetch(url)
    if request_bhav.status_code == 200:
        bhav_df = pd.read_csv(BytesIO(request_bhav.content))
    else:
        raise FileNotFoundError(f' Data not found, change the trade_date...')
    bhav_df.columns = [name.replace(' ', '') for name in bhav_df.columns]
    return bhav_df


def equity_list():
    """
    get list of all equity available to trade in NSE
    :return: pandas data frame
    """
    origin_url = "https://nsewebsite-staging.nseindia.com"
    url = "https://archives.nseindia.com/content/equities/EQUITY_L.csv"
    file_chk = nse_urlfetch(url, origin_url=origin_url)
    if file_chk.status_code != 200:
        raise FileNotFoundError(f" No data equity list available")
    try:
        data_df = pd.read_csv(BytesIO(file_chk.content))
    except Exception as e:
        raise FileNotFoundError(f' Equity List not found :: NSE error : {e}')
    data_df = data_df[['SYMBOL', 'NAME OF COMPANY', ' SERIES', ' DATE OF LISTING', ' FACE VALUE']]
    return data_df


def fno_equity_list():
    """
    get a dataframe of all listed derivative equity list with the recent lot size to trade
    :return: pandas data frame
    """
    origin_url = "https://www.nseindia.com/products-services/equity-derivatives-list-underlyings-information"
    url = "https://www.nseindia.com/api/underlying-information"
    data_obj = nse_urlfetch(url, origin_url=origin_url)
    if data_obj.status_code != 200:
        raise NSEdataNotFound(f" Resource not available for fno_equity_list")
    data_dict = data_obj.json()
    data_df = pd.DataFrame(data_dict['data']['UnderlyingList'])
    return data_df


def fno_index_list():
    """
    get a dataframe of all listed derivative index list with the recent lot size to trade
    :return: pandas data frame
    """
    origin_url = "https://www.nseindia.com/products-services/equity-derivatives-list-underlyings-information"
    url = "https://www.nseindia.com/api/underlying-information"
    data_obj = nse_urlfetch(url, origin_url=origin_url)
    if data_obj.status_code != 200:
        raise NSEdataNotFound(f" Resource not available for fno_equity_index_list")
    data_dict = data_obj.json()
    data_df = pd.DataFrame(data_dict['data']['IndexList'])
    return data_df


def nifty50_equity_list():
    """
    list of all equities under NIFTY 50 index
    :return: pandas data frame
    """
    url = "https://nsearchives.nseindia.com/content/indices/ind_nifty50list.csv"
    file_chk = nse_urlfetch(url)
    if file_chk.status_code != 200:
        raise FileNotFoundError(f" No data equity list available")
    try:
        data_df = pd.read_csv(BytesIO(file_chk.content))
    except Exception as e:
        raise FileNotFoundError(f' equities under NIFTY 50 index not found :: NSE error : {e}')
    data_df = data_df[['Company Name', 'Industry', 'Symbol']]
    return data_df


def niftynext50_equity_list():
    """
    list of all equities under NIFTY NEXT 50 index
    :return: pandas data frame
    """
    try:
        data_df = pd.read_csv("https://archives.nseindia.com/content/indices/ind_niftynext50list.csv")
    except Exception as e:
        raise FileNotFoundError(f' equities under NIFTY NEXT 50 index not found :: NSE error : {e}')
    data_df = data_df[['Company Name', 'Industry', 'Symbol']]
    return data_df


def niftymidcap150_equity_list():
    """
    list of all equities under NIFTY MIDCAP 150 index
    :return: pandas data frame
    """
    try:
        data_df = pd.read_csv("https://archives.nseindia.com/content/indices/ind_niftymidcap150list.csv")
    except Exception as e:
        raise FileNotFoundError(f' equities under NIFTY MIDCAP 150 index not found :: NSE error : {e}')
    data_df = data_df[['Company Name', 'Industry', 'Symbol']]
    return data_df


def niftysmallcap250_equity_list():
    """
    list of all equities under NIFTY SMALLCAP 250 index
    :return: pandas data frame
    """
    try:
        data_df = pd.read_csv("https://archives.nseindia.com/content/indices/ind_niftysmallcap250list.csv")
    except Exception as e:
        raise FileNotFoundError(f' equities under NIFTY SMALLCAP 250 index not found :: NSE error : {e}')
    data_df = data_df[['Company Name', 'Industry', 'Symbol']]
    return data_df


def market_watch_all_indices():
    """
    Market Watch - Indices of the day in data frame
    :return: pd.DataFrame
    """
    origin_url = "https://nsewebsite-staging.nseindia.com"
    url = "https://www.nseindia.com/api/allIndices"
    data_json = nse_urlfetch(url, origin_url=origin_url).json()
    data_df = pd.DataFrame(data_json['data'])
    print(data_df.columns)
    return data_df[['key', 'index', 'indexSymbol', 'last', 'variation', 'percentChange', 'open', 'high', 'low',
                    'previousClose', 'yearHigh', 'yearLow', 'pe', 'pb', 'dy', 'declines', 'advances', 'unchanged',
                    'perChange365d', 'perChange30d', 'previousDay', 'oneWeekAgoVal', 'oneMonthAgoVal', 'oneYearAgoVal']]


def fii_dii_trading_activity():
    """
    FII and DII trading activity of the day in data frame
    :return: pd.DataFrame
    """
    url = "https://www.nseindia.com/api/fiidiiTradeReact"
    data_json = nse_urlfetch(url).json()
    data_df = pd.DataFrame(data_json)
    return data_df


def var_begin_day(trade_date: str):
    """
    get the VaR Begin Day data as per the traded date
    :param trade_date: eg:'20-06-2023'
    :return: pandas data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    use_date = trade_date.strftime(ddmmyyyy)
    url = f'https://nsearchives.nseindia.com/archives/nsccl/var/C_VAR1_{use_date}_1.DAT'
    request_nse = nse_urlfetch(url)
    if request_nse.status_code == 200:
        var_df = pd.read_csv(BytesIO(request_nse.content), skiprows=1)
    else:
        raise FileNotFoundError(f' Data not found, change the trade_date...')
    var_df.columns = var_columns
    return var_df


def var_1st_intra_day(trade_date: str):
    """
    get the VaR 1st Intra Day data as per the traded date
    :param trade_date: eg:'20-06-2023'
    :return: pandas data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    use_date = trade_date.strftime(ddmmyyyy)
    url = f'https://nsearchives.nseindia.com/archives/nsccl/var/C_VAR1_{use_date}_2.DAT'
    request_nse = nse_urlfetch(url)
    if request_nse.status_code == 200:
        var_df = pd.read_csv(BytesIO(request_nse.content), skiprows=1)
    else:
        raise FileNotFoundError(f' Data not found, change the trade_date...')
    var_df.columns = var_columns
    return var_df


def var_2nd_intra_day(trade_date: str):
    """
    get the VaR 2nd Intra Day data as per the traded date
    :param trade_date: eg:'20-06-2023'
    :return: pandas data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    use_date = trade_date.strftime(ddmmyyyy)
    url = f'https://nsearchives.nseindia.com/archives/nsccl/var/C_VAR1_{use_date}_3.DAT'
    request_nse = nse_urlfetch(url)
    if request_nse.status_code == 200:
        var_df = pd.read_csv(BytesIO(request_nse.content), skiprows=1)
    else:
        raise FileNotFoundError(f' Data not found, change the trade_date...')
    var_df.columns = var_columns
    return var_df


def var_3rd_intra_day(trade_date: str):
    """
    get the VaR 3rd Intra Day data as per the traded date
    :param trade_date: eg:'20-06-2023'
    :return: pandas data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    use_date = trade_date.strftime(ddmmyyyy)
    url = f'https://nsearchives.nseindia.com/archives/nsccl/var/C_VAR1_{use_date}_4.DAT'
    request_nse = nse_urlfetch(url)
    if request_nse.status_code == 200:
        var_df = pd.read_csv(BytesIO(request_nse.content), skiprows=1)
    else:
        raise FileNotFoundError(f' Data not found, change the trade_date...')
    var_df.columns = var_columns
    return var_df


def var_4th_intra_day(trade_date: str):
    """
    get the VaR 4th Intra Day data as per the traded date
    :param trade_date: eg:'20-06-2023'
    :return: pandas data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    use_date = trade_date.strftime(ddmmyyyy)
    url = f'https://nsearchives.nseindia.com/archives/nsccl/var/C_VAR1_{use_date}_5.DAT'
    request_nse = nse_urlfetch(url)
    if request_nse.status_code == 200:
        var_df = pd.read_csv(BytesIO(request_nse.content), skiprows=1)
    else:
        raise FileNotFoundError(f' Data not found, change the trade_date...')
    var_df.columns = var_columns
    return var_df


def var_end_of_day(trade_date: str):
    """
    get the VaR End of Day data as per the traded date
    :param trade_date: eg:'20-06-2023'
    :return: pandas data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    use_date = trade_date.strftime(ddmmyyyy)
    url = f'https://nsearchives.nseindia.com/archives/nsccl/var/C_VAR1_{use_date}_6.DAT'
    request_nse = nse_urlfetch(url)
    if request_nse.status_code == 200:
        var_df = pd.read_csv(BytesIO(request_nse.content), skiprows=1)
    else:
        raise FileNotFoundError(f' Data not found, change the trade_date...')
    var_df.columns = var_columns
    return var_df


def sme_bhav_copy(trade_date: str):
    """
    get the SME bhav copy data as per the traded date
    :param trade_date: eg:'20-06-2023'
    :return: pandas data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    use_date = trade_date.strftime(ddmmyy)
    url = f'https://nsearchives.nseindia.com/archives/sme/bhavcopy/sme{use_date}.csv'
    request_bhav = nse_urlfetch(url)
    if request_bhav.status_code == 200:
        bhav_df = pd.read_csv(BytesIO(request_bhav.content))
    else:
        raise FileNotFoundError(f' Data not found, change the trade_date...')
    bhav_df.columns = [name.replace(' ', '') for name in bhav_df.columns]
    return bhav_df


def sme_band_complete(trade_date: str):
    """
    get the SME Band Complete data as per the traded date
    :param trade_date: eg:'20-06-2023'
    :return: pandas data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    use_date = trade_date.strftime(ddmmyyyy)
    url = f'https://nsearchives.nseindia.com/sme/content/price_band/archieves/sme_bands_complete_{use_date}.csv'
    request_sme = nse_urlfetch(url)
    if request_sme.status_code == 200:
        sme_df = pd.read_csv(BytesIO(request_sme.content))
    else:
        raise FileNotFoundError(f' Data not found, change the trade_date...')
    sme_df.columns = [name.replace(' ', '') for name in sme_df.columns]
    return sme_df


def week_52_high_low_report(trade_date: str):
    """
    get the 52-Week High Low Report data as per the traded date
    :param trade_date: eg:'20-06-2023'
    :return: pandas data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    use_date = trade_date.strftime(ddmmyyyy)
    url = f'https://nsearchives.nseindia.com/content/CM_52_wk_High_low_{use_date}.csv'
    request_nse = nse_urlfetch(url)
    if request_nse.status_code == 200:
        high_low_df = pd.read_csv(BytesIO(request_nse.content), skiprows=2)
    else:
        raise FileNotFoundError(f' Data not found, change the trade_date...')
    high_low_df.columns = [name.replace(' ', '') for name in high_low_df.columns]
    return high_low_df


def financial_results_for_equity(from_date: str = None,
                                 to_date: str = None,
                                 period: str = None,
                                 fo_sec: bool = False,
                                 fin_period: str = 'Quarterly'):

    """
    get audited and un-auditable financial results for equities. as per
    https://www.nseindia.com/companies-listing/corporate-filings-financial-results
    :param fin_period: Quaterly/ Half-Yearly/ Annual/ Others
    :param fo_sec: True/False
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,
                            '1W': for last 7 days data,
                            '1M': from last month same date,
                            '6M': last 6 month data,
                            '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    master_data_df, headers, ns, keys_to_extract = get_financial_results_master(from_date, to_date, period,
                                                                                fo_sec, fin_period)
    fin_df, df = pd.DataFrame(), pd.DataFrame()
    for row in master_data_df.itertuples(index=False):
        try:
            # Fetch the XML content
            response_ = requests.get(row.xbrl, headers=headers)
            response_.raise_for_status()  # Check for HTTP errors

            # Parse the XML content
            root = ET.fromstring(response_.content)

            # Extract values
            extracted_data = {}
            for key in keys_to_extract:
                elem_ = root.find(f".//in-bse-fin:{key}", ns)
                extracted_data[key] = elem_.text if elem_ is not None else None

            # Convert to Pandas DataFrame
            df = pd.DataFrame([extracted_data])
        except requests.exceptions.RequestException as e:
            raise requests.exceptions.RequestException(f"Request failed: {e}") from e
        except ET.ParseError as e:
            raise ET.ParseError(f"XML parsing failed: {e}") from e
        except Exception as e:
            raise RuntimeError(f"An error occurred: {e}") from e

        if fin_df.empty:
            fin_df = df
        else:
            fin_df = pd.concat([fin_df, df], ignore_index=True)
    return fin_df


def corporate_bond_trade_report(trade_date: str):
    """
    get the NSE corporate bond trade report as per the traded date
    :param trade_date: eg:'20-06-2023'
    :return: pandas data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    use_date = trade_date.strftime(ddmmyy)
    url = f'https://nsearchives.nseindia.com/archives/equities/corpbond/corpbond{use_date}.csv'
    request_bhav = nse_urlfetch(url)
    if request_bhav.status_code == 200:
        bond_df = pd.read_csv(BytesIO(request_bhav.content))
    else:
        raise FileNotFoundError(f' Data not found, change the trade_date...')
    bond_df.columns = [name.replace(' ', '') for name in bond_df.columns]
    bond_df['SERIES'] = bond_df['SERIES'].str.replace(' ', '')
    return bond_df


def pe_ratio(trade_date: str):
    """
    get the NSE pe ratio for all NSE equities data as per the traded date
    :param trade_date: eg:'20-06-2023'
    :return: pandas data frame
    """
    trade_date = datetime.strptime(trade_date, dd_mm_yyyy)
    use_date = trade_date.strftime(ddmmyy)
    url = f'https://nsearchives.nseindia.com/content/equities/peDetail/PE_{use_date}.csv'
    request_bhav = nse_urlfetch(url)
    if request_bhav.status_code == 200:
        pe_df = pd.read_csv(BytesIO(request_bhav.content))
    else:
        raise FileNotFoundError(f' Data not found, change the trade_date...')
    pe_df.columns = [name.replace(' ', '') for name in pe_df.columns]
    return pe_df


def corporate_actions_for_equity(from_date: str = None,
                                 to_date: str = None,
                                 period: str = None,
                                 fno_only: bool = False):

    """
    get corporate actions for equities as per
    https://www.nseindia.com/companies-listing/corporate-filings-actions
    :param fno_only: True/False
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,
                            '1W': for last 7 days data,
                            '1M': from last month same date,
                            '6M': last 6 month data,
                            '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    origin_url = "https://www.nseindia.com/companies-listing/corporate-filings-actions"
    url_ = "https://www.nseindia.com/api/corporates-corporateactions?index=equities&"
    if fno_only:
        payload = f'from_date={from_date}&to_date={to_date}&fo_sec=true'
    else:
        payload = f'from_date={from_date}&to_date={to_date}'
    data_text = nse_urlfetch(url_ + payload, origin_url=origin_url)
    if data_text.status_code != 200:
        raise NSEdataNotFound(f" Resource not available for financial data with these parameters")
    json_str = data_text.content.decode("utf-8")
    data_list = json.loads(json_str)
    master_data_df = pd.DataFrame(data_list)
    master_data_df.columns = [name.replace(' ', '') for name in master_data_df.columns]
    return master_data_df


def event_calendar_for_equity(from_date: str = None,
                              to_date: str = None,
                              period: str = None,
                              fno_only: bool = False):

    """
    get event calendar for equities as per
    https://www.nseindia.com/companies-listing/corporate-filings-event-calendar
    :param fno_only: True/False
    :param from_date: '17-03-2022' ('dd-mm-YYYY')
    :param to_date: '17-06-2023' ('dd-mm-YYYY')
    :param period: use one {'1D': last day data,
                            '1W': for last 7 days data,
                            '1M': from last month same date,
                            '6M': last 6 month data,
                            '1Y': from last year same date)
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    validate_date_param(from_date, to_date, period)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    origin_url = "https://www.nseindia.com/companies-listing/corporate-filings-event-calendar"
    url_ = "https://www.nseindia.com/api/event-calendar?index=equities&"
    if fno_only:
        payload = f'from_date={from_date}&to_date={to_date}&fo_sec=true'
    else:
        payload = f'from_date={from_date}&to_date={to_date}'
    data_text = nse_urlfetch(url_ + payload, origin_url=origin_url)
    if data_text.status_code != 200:
        raise NSEdataNotFound(f" Resource not available for financial data with these parameters")
    json_str = data_text.content.decode("utf-8")
    data_list = json.loads(json_str)
    master_data_df = pd.DataFrame(data_list)
    master_data_df.columns = [name.replace(' ', '') for name in master_data_df.columns]
    return master_data_df


def top_gainers_or_losers(to_get: str = 'gainers'):
    """
    get top gainers or losers on live market, after market hour it will get as per last traded value
    :param to_get: gainers/loosers
    :return: pandas.DataFrame
    :raise ValueError if the parameter input is not proper
    """
    static_options_list = ['gainers', 'loosers']
    validate_param_from_list(to_get, static_options_list)
    data_json = get_top_gainers_or_losers(to_get)
    legends_dict = {item[0]: item[1] for item in data_json['legends']}
    gainers_losers_df = pd.DataFrame()
    for i in legends_dict.keys():
        data_df = pd.DataFrame(data_json[i]['data'])
        data_df['legend'] = i
        if gainers_losers_df.empty:
            gainers_losers_df = data_df
        else:
            gainers_losers_df = pd.concat([gainers_losers_df, data_df])
    return gainers_losers_df


def most_active_equities(fetch_by: str = 'value'):
    """
    to get most active equities fetched by value/volume in live market, after market hour it will get as per last traded value
    link : https://www.nseindia.com/market-data/most-active-equities
    :param fetch_by: select any volume/value
    :return: pandas dataframe
    """
    static_options_list = ['volume', 'value']
    validate_param_from_list(fetch_by, static_options_list)
    origin_url = "https://www.nseindia.com/market-data/most-active-equities"
    url = f"https://www.nseindia.com/api/live-analysis-most-active-securities?index={fetch_by}"
    try:
        data_json = nse_urlfetch(url, origin_url=origin_url).json()
        data_df = pd.DataFrame(data_json['data'])
    except Exception as e:
        raise NSEdataNotFound(f" Resource not available MSG: {e}")
    return data_df


def total_traded_stocks():
    """
    to get all total traded stocks detail in live market, after market hour it will get as per last traded value
    summary_dict - has the live market summary for the stocks in dictionary format
    detail_df - has all detail available in the current market in dataframe format
    link : https://www.nseindia.com/market-data/stocks-traded
    :return: summary_dict, detail_df
    """
    origin_url = "https://www.nseindia.com/market-data/stocks-traded"
    url = f"https://www.nseindia.com/api/live-analysis-stocksTraded"
    try:
        data_json = nse_urlfetch(url, origin_url=origin_url).json()
        summary_dict = data_json['total']['count']
        detail_df = pd.DataFrame(data_json['total']['data'])
    except Exception as e:
        raise NSEdataNotFound(f" Resource not available MSG: {e}")
    return summary_dict, detail_df


# if __name__ == '__main__':
    # data = pe_ratio(trade_date='11-09-2024')  # trade_date='11-09-2024'
    # data = index_data(index='NIFTY 50', period='1W')
    # data = block_deals_data(period='1W')
    # data = bulk_deal_data(period='1W')
    # data = india_vix_data(period='1W')
    # data = short_selling_data(period='1W')
    # data = index_data(index='NIFTY 50', from_date='21-10-2024', to_date='30-10-2024')
    # data = deliverable_position_data(symbol='SBIN', from_date='23-03-2024', to_date='23-06-2024')
    # data = market_watch_all_indices()
    # data = fno_equity_list()
    # data = price_volume_and_deliverable_position_data(symbol='SBIN',  from_date='23-03-2024', to_date='23-06-2024')
    # data = price_volume_and_deliverable_position_data(symbol='SBIN', period='1M')
    # data = price_volume_data(symbol='SBIN', from_date='20-06-2023', to_date='20-07-2023')
    # data = financial_results_for_equity(from_date='11-03-2025', to_date='16-03-2025', fo_sec=True,
    #                                     fin_period='Quarterly')
    # data = corporate_actions_for_equity(period='6M', fno_only=False)
    # data = event_calendar_for_equity(period='1M', fno_only=False)
    # data = sme_band_complete(trade_date='11-03-2025')
    # data = top_gainers_or_losers('loosers')   # gainers/losers
    # data = most_active_equities(fetch_by='volume')  # value/volume
    # data = total_traded_stocks()
    # df = index_data("NIFTY 50", from_date="01-11-2024", to_date="27-12-2024")
    # print(f"Success! Got {df} rows")
    # data.to_csv(fr'C:\Ruchi Tanmay\Stock Market\Data Analysis\Final Data\data.csv')

    # data = niftymidcap150_equity_list()
    # print(data)
    # print(data.info())
    # -----------------------------------------------------




from io import BytesIO, StringIO
import json
from nselib.libutil import *
from nselib.constants import *


def get_price_volume_and_deliverable_position_data(symbol: str, from_date: str, to_date: str):
    origin_url = "https://nsewebsite-staging.nseindia.com/report-detail/eq_security"
    url = "https://www.nseindia.com/api/historicalOR/generateSecurityWiseHistoricalData?"
    payload = f"from={from_date}&to={to_date}&symbol={symbol}&type=priceVolumeDeliverable&series=ALL&csv=true"
    try:
        data_text = nse_urlfetch(url + payload, origin_url=origin_url).text
        data_text = data_text.replace('\x82', '').replace('â¹', 'In Rs')
        with open('file.csv', 'w') as f:
            f.write(data_text)
        f.close()
    except Exception as e:
        raise NSEdataNotFound(f" Resource not available MSG: {e}")
    data_df = pd.read_csv('file.csv')
    data_df.columns = [name.replace(' ', '') for name in data_df.columns]
    return data_df


def get_price_volume_data(symbol: str, from_date: str, to_date: str):
    origin_url = "https://nsewebsite-staging.nseindia.com/report-detail/eq_security"
    url = "https://www.nseindia.com/api/historicalOR/generateSecurityWiseHistoricalData?"
    payload = f"from={from_date}&to={to_date}&symbol={symbol}&type=priceVolume&series=ALL&csv=true"
    try:
        data_text = nse_urlfetch(url + payload, origin_url=origin_url)
        if data_text.status_code != 200:
            raise NSEdataNotFound(f" Resource not available for Price Volume Data")
    except Exception as e:
        raise NSEdataNotFound(f" Resource not available MSG: {e}")
    data_df = pd.read_csv(BytesIO(data_text.content), index_col=False)
    data_df.columns = [name.replace(' ', '') for name in data_df.columns]
    return data_df


def get_deliverable_position_data(symbol: str, from_date: str, to_date: str):
    origin_url = "https://nsewebsite-staging.nseindia.com/report-detail/eq_security"
    url = "https://www.nseindia.com/api/historicalOR/generateSecurityWiseHistoricalData?"
    payload = f"from={from_date}&to={to_date}&symbol={symbol}&type=deliverable&series=ALL&csv=true"
    try:
        data_text = nse_urlfetch(url + payload, origin_url=origin_url)
        if data_text.status_code != 200:
            raise NSEdataNotFound(f" Resource not available for deliverable_position_data")
    except Exception as e:
        raise NSEdataNotFound(f" Resource not available MSG: {e}")
    data_df = pd.read_csv(BytesIO(data_text.content), index_col=False)
    data_df.columns = [name.replace(' ', '') for name in data_df.columns]
    return data_df


def get_india_vix_data(from_date: str, to_date: str):
    origin_url = "https://nsewebsite-staging.nseindia.com/report-detail/eq_security"
    url = f"https://www.nseindia.com/api/historicalOR/vixhistory?from={from_date}&to={to_date}&csv=true"
    try:
        data_json = nse_urlfetch(url, origin_url=origin_url).json()
        data_df = pd.DataFrame(data_json['data'])
    except Exception as e:
        raise NSEdataNotFound(f" Resource not available MSG: {e}")
    # data_df.drop(columns='TIMESTAMP', inplace=True)
    data_df.columns = cleaning_column_name(data_df.columns)
    return data_df[india_vix_data_column]


def get_index_data(index: str, from_date: str, to_date: str):
    index = index.replace(' ', '%20').upper()
    origin_url = "https://www.nseindia.com/reports-indices-historical-index-data"
    url = f"https://www.nseindia.com/api/historicalOR/indicesHistory?indexType={index}&from={from_date}&to={to_date}"
    try:
        data_json = nse_urlfetch(url, origin_url=origin_url).json()
        data_df = pd.DataFrame(data_json['data'])
    except Exception as e:
        raise NSEdataNotFound(f" Resource not available MSG: {e}")
    data_df.drop(columns='HI_TIMESTAMP', inplace=True)
    data_df.columns = index_data_columns
    return data_df


def get_bulk_deal_data(from_date: str, to_date: str):
    # print(from_date, to_date)
    origin_url = "https://nsewebsite-staging.nseindia.com"
    url = "https://www.nseindia.com/api/historicalOR/bulk-block-short-deals?optionType=bulk_deals&"
    payload = f"from={from_date}&to={to_date}&csv=true"
    # print(url + payload)
    data_text = nse_urlfetch(url + payload, origin_url=origin_url)
    if data_text.status_code != 200:
        raise NSEdataNotFound(f" Resource not available for bulk_deal_data")
    data_df = pd.read_csv(BytesIO(data_text.content), index_col=False)
    data_df.columns = [name.replace(' ', '') for name in data_df.columns]
    return data_df


def get_block_deals_data(from_date: str, to_date: str):
    # print(from_date, to_date)
    origin_url = "https://nsewebsite-staging.nseindia.com"
    url = "https://www.nseindia.com/api/historicalOR/bulk-block-short-deals?optionType=block_deals&"
    payload = f"from={from_date}&to={to_date}&csv=true"
    data_text = nse_urlfetch(url + payload, origin_url=origin_url)
    if data_text.status_code != 200:
        raise NSEdataNotFound(f" Resource not available for block_deals_data")
    data_df = pd.read_csv(BytesIO(data_text.content), index_col=False)
    data_df.columns = [name.replace(' ', '') for name in data_df.columns]
    return data_df


def get_short_selling_data(from_date: str, to_date: str):
    """
    NSE short selling data in data frame
    :param from_date:
    :param to_date:
    :return:
    """
    # print(from_date, to_date)
    origin_url = "https://nsewebsite-staging.nseindia.com"
    url = "https://www.nseindia.com/api/historicalOR/bulk-block-short-deals?optionType=short_selling&"
    payload = f"from={from_date}&to={to_date}&csv=true"
    data_text = nse_urlfetch(url + payload,origin_url=origin_url)
    if data_text.status_code != 200:
        raise NSEdataNotFound(f" Resource not available for short_selling_data")
    data_df = pd.read_csv(BytesIO(data_text.content), index_col=False)
    data_df.columns = [name.replace(' ', '') for name in data_df.columns]
    return data_df


def get_financial_results_master(from_date: str = None,
                                 to_date: str = None,
                                 period: str = None,
                                 fo_sec: bool = False,
                                 fin_period: str = 'Quarterly'):
    validate_date_param(from_date, to_date, period)
    from_date, to_date = derive_from_and_to_date(from_date=from_date, to_date=to_date, period=period)
    origin_url = "https://www.nseindia.com/companies-listing/corporate-filings-financial-results"
    url_ = "https://www.nseindia.com/api/corporates-financial-results?index=equities&"
    if fo_sec:
        payload = f'from_date={from_date}&to_date={to_date}&fo_sec=true&period={fin_period}'
    else:
        payload = f'from_date={from_date}&to_date={to_date}&period={fin_period}'
    data_text = nse_urlfetch(url_ + payload, origin_url=origin_url)
    if data_text.status_code != 200:
        raise NSEdataNotFound(f" Resource not available for financial data with these parameters")
    json_str = data_text.content.decode("utf-8")
    data_list = json.loads(json_str)
    master_data_df = pd.DataFrame(data_list)
    master_data_df.columns = [name.replace(' ', '') for name in master_data_df.columns]
    headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3',
            'Accept': '*/*',
            'Accept-Language': 'en-US,en;q=0.9',
            'Referer': 'https://www.nseindia.com/'
        }
    ns = {
            "xbrli": "http://www.xbrl.org/2003/instance",
            "in-bse-fin": "http://www.bseindia.com/xbrl/fin/2020-03-31/in-bse-fin"
        }
    keys_to_extract = [
        "ScripCode", "Symbol", "MSEISymbol", "NameOfTheCompany", "ClassOfSecurity",
        "DateOfStartOfFinancialYear", "DateOfEndOfFinancialYear",
        "DateOfBoardMeetingWhenFinancialResultsWereApproved",
        "DateOnWhichPriorIntimationOfTheMeetingForConsideringFinancialResultsWasInformedToTheExchange",
        "DescriptionOfPresentationCurrency", "LevelOfRoundingUsedInFinancialStatements",
        "ReportingQuarter", "StartTimeOfBoardMeeting", "EndTimeOfBoardMeeting",
        "DateOfStartOfBoardMeeting", "DateOfEndOfBoardMeeting",
        "DeclarationOfUnmodifiedOpinionOrStatementOnImpactOfAuditQualification",
        "IsCompanyReportingMultisegmentOrSingleSegment", "DescriptionOfSingleSegment",
        "DateOfStartOfReportingPeriod", "DateOfEndOfReportingPeriod",
        "WhetherResultsAreAuditedOrUnaudited", "NatureOfReportStandaloneConsolidated",
        "RevenueFromOperations", "OtherIncome", "Income", "CostOfMaterialsConsumed",
        "PurchasesOfStockInTrade", "ChangesInInventoriesOfFinishedGoodsWorkInProgressAndStockInTrade",
        "EmployeeBenefitExpense", "FinanceCosts", "DepreciationDepletionAndAmortisationExpense",
        "OtherExpenses", "Expenses", "ProfitBeforeExceptionalItemsAndTax", "ExceptionalItemsBeforeTax",
        "ProfitBeforeTax", "CurrentTax", "DeferredTax", "TaxExpense",
        "NetMovementInRegulatoryDeferralAccountBalancesRelatedToProfitOrLossAndTheRelatedDeferredTaxMovement",
        "ProfitLossForPeriodFromContinuingOperations", "ProfitLossFromDiscontinuedOperationsBeforeTax",
        "TaxExpenseOfDiscontinuedOperations", "ProfitLossFromDiscontinuedOperationsAfterTax",
        "ShareOfProfitLossOfAssociatesAndJointVenturesAccountedForUsingEquityMethod",
        "ProfitLossForPeriod", "OtherComprehensiveIncomeNetOfTaxes",
        "ComprehensiveIncomeForThePeriod", "ProfitOrLossAttributableToOwnersOfParent",
        "ProfitOrLossAttributableToNonControllingInterests",
        "ComprehensiveIncomeForThePeriodAttributableToOwnersOfParent",
        "ComprehensiveIncomeForThePeriodAttributableToOwnersOfParentNonControllingInterests",
        "PaidUpValueOfEquityShareCapital", "FaceValueOfEquityShareCapital",
        "BasicEarningsLossPerShareFromContinuingOperations",
        "DilutedEarningsLossPerShareFromContinuingOperations",
        "BasicEarningsLossPerShareFromDiscontinuedOperations",
        "DilutedEarningsLossPerShareFromDiscontinuedOperations",
        "BasicEarningsLossPerShareFromContinuingAndDiscontinuedOperations",
        "DilutedEarningsLossPerShareFromContinuingAndDiscontinuedOperations",
        "DescriptionOfOtherExpenses", "OtherExpenses",
        "DescriptionOfItemThatWillNotBeReclassifiedToProfitAndLoss",
        "AmountOfItemThatWillNotBeReclassifiedToProfitAndLoss",
        "IncomeTaxRelatingToItemsThatWillNotBeReclassifiedToProfitOrLoss",
        "DescriptionOfItemThatWillBeReclassifiedToProfitAndLoss",
        "AmountOfItemThatWillBeReclassifiedToProfitAndLoss",
        "IncomeTaxRelatingToItemsThatWillBeReclassifiedToProfitOrLoss"
    ]
    return master_data_df, headers, ns, keys_to_extract


def get_top_gainers_or_losers(to_get: str):
    origin_url = "https://www.nseindia.com/market-data/top-gainers-losers"
    url = f"https://www.nseindia.com/api/live-analysis-variations?index={to_get}"
    try:
        data_json = nse_urlfetch(url, origin_url=origin_url).json()
    except Exception as e:
        raise NSEdataNotFound(f" Resource not available MSG: {e}")
    return data_json



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='nselib',
    packages=setuptools.find_packages(),
    version='2.4.2',
    include_package_data=True,
    description='library to get NSE India data',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Ruchi Tanmay, Amlan Mishra',
    author_email='ruchitanmay@gmail.com',
    url='https://github.com/RuchiTanmay/nselib',
    install_requires=['requests', 'pandas', 'scipy', 'pandas_market_calendars'],
    keywords=['nseindia', 'nse', 'nse data', 'stock data', 'python', 'nse daily data', 'stock markets',
              'nse library', 'nse python', 'nse daily reports'],
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



import os
from datetime import datetime, timedelta, date
import requests
import numpy as np
import pandas as pd
from nselib.constants import *
import pandas_market_calendars as mcal

default_header = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/129.0.0.0 Safari/537.36"
}

header = {
            "referer": "https://www.nseindia.com/",
             "Connection": "keep-alive",
             "Cache-Control": "max-age=0",
             "DNT": "1",
             "Upgrade-Insecure-Requests": "1",
             "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/130.0.0.0 Safari/537.36",
             "Sec-Fetch-User": "?1",
             "Accept": "ext/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
             "Sec-Fetch-Site": "none",
             "Sec-Fetch-Mode": "navigate",
             "Accept-Language": "en-US,en;q=0.9,hi;q=0.8"
            }

nse_calendar = mcal.get_calendar("NSE")


class CalenderNotFound(Exception):
    def __init__(self, message):

        # Call the base class constructor with the parameters it needs
        super(CalenderNotFound, self).__init__(message)


class NSEdataNotFound(Exception):
    def __init__(self, message):
        super(NSEdataNotFound, self).__init__(message)


def validate_param_from_list(value: str, static_options_list: list):
    if value not in static_options_list:
        raise ValueError(f"'{value}' not a valid parameter :: select from {static_options_list}")
    else:
        pass


def validate_date_param(from_date: str, to_date: str, period: str):
    if not period and (not from_date or not to_date):
        raise ValueError(' Please provide the valid parameters')
    elif period and period.upper() not in equity_periods:
        raise ValueError(f'period = {period} is not a valid value')

    try:
        if not period:
            from_date = datetime.strptime(from_date, dd_mm_yyyy)
            to_date = datetime.strptime(to_date, dd_mm_yyyy)
            time_delta = (to_date - from_date).days
            if time_delta < 1:
                raise ValueError(f'to_date should greater than from_date ')
    except Exception as e:
        print(e)
        raise ValueError(f'either or both from_date = {from_date} || to_date = {to_date} are not valid value')


def subtract_months(dt, months):
    # calculate target year and month
    year = dt.year
    month = dt.month - months

    # adjust year and month when month < 1
    while month <= 0:
        month += 12
        year -= 1

    # maximum days in each month
    month_days = [31,
                  29 if (year % 4 == 0 and (year % 100 != 0 or year % 400 == 0)) else 28,
                  31, 30, 31, 30,
                  31, 31, 30, 31, 30, 31]

    # adjust the day if current date > max days in target month
    day = min(dt.day, month_days[month - 1])

    return date(year, month, day)


def derive_from_and_to_date(from_date: str = None, to_date: str = None, period: str = None):
    if not period:
        return from_date, to_date
    today = date.today()
    conditions = [period.upper() == '1D',
                  period.upper() == '1W',
                  period.upper() == '1M',
                  period.upper() == '6M',
                  period.upper() == '1Y'
                  ]
    value = [today - timedelta(days=1),
             today - timedelta(weeks=1),
             subtract_months(today, 1),
             subtract_months(today, 6),
             subtract_months(today, 12)]

    f_date = np.select(conditions, value, default=(today - timedelta(days=1)))
    f_date = pd.to_datetime(str(f_date))
    while True:
        date_chk = nse_calendar.schedule(start_date=f_date, end_date=f_date)
        if not date_chk.empty:  # If market was open on this day
            break  # Stop the loop
        f_date -= timedelta(days=1)
    from_date = f_date.strftime(dd_mm_yyyy)
    today = today.strftime(dd_mm_yyyy)
    return from_date, today


def cleaning_column_name(col: list):
    unwanted_str_list = ['FH_', 'EOD_', 'HIT_']
    new_col = col
    for unwanted in unwanted_str_list:
        new_col = [name.replace(f'{unwanted}', '') for name in new_col]
    return new_col


def cleaning_nse_symbol(symbol):
    symbol = symbol.replace('&', '%26')  # URL Parse for Stocks Like M&M Finance
    return symbol.upper()


def nse_urlfetch(url, origin_url="http://nseindia.com"):
    r_session = requests.session()
    nse_live = r_session.get(origin_url, headers=default_header)
    cookies = nse_live.cookies
    return r_session.get(url, headers=header, cookies=cookies)


def get_nselib_path():
    """
    Extract isap file path
    """
    mydir = os.getcwd()
    return mydir.split(r'\nselib', 1)[0]


def get_month_from_date(trade_date):
    """
    get the month
    :param trade_date:
    :return: 'Dec'
    """
    return datetime.strptime(trade_date, '%Y-%m-%d').strftime('%b')


def trading_holiday_calendar():
    data_df = pd.DataFrame(columns=['Product', 'tradingDate', 'weekDay', 'description', 'Sr_no'])
    url = "https://www.nseindia.com/api/holiday-master?type=trading"
    try:
        data_dict = nse_urlfetch(url).json()
    except Exception as e:
        raise CalenderNotFound(" Calender data Not found try after some time ")
    for prod in data_dict:
        h_df = pd.DataFrame(data_dict[prod])
        h_df['Product'] = prod
        data_df = pd.concat([data_df, h_df])
    condition = [data_df['Product'] == 'CBM', data_df['Product'] == 'CD', data_df['Product'] == 'CM',
                 data_df['Product'] == 'CMOT', data_df['Product'] == 'COM', data_df['Product'] == 'FO',
                 data_df['Product'] == 'IRD', data_df['Product'] == 'MF', data_df['Product'] == 'NDM',
                 data_df['Product'] == 'NTRP', data_df['Product'] == 'SLBS']
    value = ['Corporate Bonds', 'Currency Derivatives', 'Equities', 'CMOT', 'Commodity Derivatives', 'Equity Derivatives',
             'Interest Rate Derivatives', 'Mutual Funds', 'New Debt Segment', 'Negotiated Trade Reporting Platform',
             'Securities Lending & Borrowing Schemes']
    data_df['Product'] = np.select(condition, value, default='Unknown')
    return data_df


if __name__ == '__main__':
    # data = derive_from_and_to_date('6M')
    print(trading_holiday_calendar())



